Package: rnaturalearth
Title: World Map Data from Natural Earth
Version: 1.2.0
Authors@R: c(
  person(
    family = "Massicotte",
    given = "Philippe",
    email = "pmassicotte@hotmail.com",
    role = c("aut", "cre"),
    comment = c(ORCID = "0000-0002-5919-4116")
  ),
  person(
    family = "South",
    given = "Andy",
    email = "southandy@gmail.com",
    role = c("aut")
  ),
  person(
    family = "Hufkens",
    given = "Koen",
    email = "koen.hufkens@gmail.com",
    role = c("ctb"),
    comment = c(ORCID = "0000-0002-5070-8109")
  ))
Description: Facilitates mapping by making natural earth map data from <https://www.naturalearthdata.com/> more easily available to R users.
License: MIT + file LICENSE
LazyData: true
LazyDataCompression: xz
Encoding: UTF-8
URL: https://docs.ropensci.org/rnaturalearth/,
        https://github.com/ropensci/rnaturalearth,
        https://docs.ropensci.org/rnaturalearthhires/
BugReports: https://github.com/ropensci/rnaturalearth/issues
Additional_repositories: https://ropensci.r-universe.dev
Depends: R (>= 3.1.1)
Imports: cli, httr (>= 1.1.0), jsonlite, sf (>= 0.3-4), terra, utils
        (>= 3.2.3)
Suggests: pak, dplyr, ggplot2, ggrepel, knitr (>= 1.12.3), lifecycle
        (>= 1.0.3), pbapply, purrr, rlang, rmarkdown, rnaturalearthdata
        (>= 1.0.0), rnaturalearthhires (>= 1.0.0), testthat (>= 0.9.1),
        tmap
Config/Needs/website: ropensci/rnaturalearthhires
VignetteBuilder: knitr
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-01-19 14:12:33 UTC; filoche
Author: Philippe Massicotte [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5919-4116>),
  Andy South [aut],
  Koen Hufkens [ctb] (ORCID: <https://orcid.org/0000-0002-5070-8109>)
Maintainer: Philippe Massicotte <pmassicotte@hotmail.com>
Repository: CRAN
Date/Publication: 2026-01-19 17:20:02 UTC
Built: R 4.6.0; ; 2026-04-26 05:25:56 UTC; unix

Package: rnaturalearth
Title: World Map Data from Natural Earth
Version: 1.2.0
Authors@R: c(
  person(
    family = "Massicotte",
    given = "Philippe",
    email = "pmassicotte@hotmail.com",
    role = c("aut", "cre"),
    comment = c(ORCID = "0000-0002-5919-4116")
  ),
  person(
    family = "South",
    given = "Andy",
    email = "southandy@gmail.com",
    role = c("aut")
  ),
  person(
    family = "Hufkens",
    given = "Koen",
    email = "koen.hufkens@gmail.com",
    role = c("ctb"),
    comment = c(ORCID = "0000-0002-5070-8109")
  ))
Description: Facilitates mapping by making natural earth map data from <https://www.naturalearthdata.com/> more easily available to R users.
License: MIT + file LICENSE
LazyData: true
LazyDataCompression: xz
Encoding: UTF-8
URL: https://docs.ropensci.org/rnaturalearth/,
        https://github.com/ropensci/rnaturalearth,
        https://docs.ropensci.org/rnaturalearthhires/
BugReports: https://github.com/ropensci/rnaturalearth/issues
Additional_repositories: https://ropensci.r-universe.dev
Depends: R (>= 3.1.1)
Imports: cli, httr (>= 1.1.0), jsonlite, sf (>= 0.3-4), terra, utils
        (>= 3.2.3)
Suggests: pak, dplyr, ggplot2, ggrepel, knitr (>= 1.12.3), lifecycle
        (>= 1.0.3), pbapply, purrr, rlang, rmarkdown, rnaturalearthdata
        (>= 1.0.0), rnaturalearthhires (>= 1.0.0), testthat (>= 0.9.1),
        tmap
Config/Needs/website: ropensci/rnaturalearthhires
VignetteBuilder: knitr
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-01-19 14:12:33 UTC; filoche
Author: Philippe Massicotte [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5919-4116>),
  Andy South [aut],
  Koen Hufkens [ctb] (ORCID: <https://orcid.org/0000-0002-5070-8109>)
Maintainer: Philippe Massicotte <pmassicotte@hotmail.com>
Repository: CRAN
Date/Publication: 2026-01-19 17:20:02 UTC

# Generated by roxygen2: do not edit by hand

export(check_data_exist)
export(check_rnaturalearthdata)
export(check_rnaturalearthhires)
export(install_rnaturalearthdata)
export(install_rnaturalearthhires)
export(ne_coastline)
export(ne_countries)
export(ne_download)
export(ne_file_name)
export(ne_find_vector_data)
export(ne_load)
export(ne_states)

<!-- README.md is generated from README.Rmd. Please edit that file -->

<!-- used devtools::build_readme() to update the md -->

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/rnaturalearth)](https://CRAN.R-project.org/package=rnaturalearth)
[![](https://badges.ropensci.org/22_status.svg)](https://github.com/ropensci/software-review/issues/22)
[![Project Status: Active – The project has reached a stable, usable
state and is being actively
developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![R-CMD-check](https://github.com/ropensci/rnaturalearth/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/ropensci/rnaturalearth/actions/workflows/R-CMD-check.yaml)

<!-- badges: end -->

# rnaturalearth

An R package to hold and facilitate interaction with [Natural
Earth](https://www.naturalearthdata.com/) map data.

This package provides :

- access to a pre-downloaded subset of Natural Earth v4.1.0 (March 2018)
  vector data commonly used in world mapping

- easy subsetting by countries and regions

- functions to download other Natural Earth vector and raster data

- a simple, reproducible and sustainable workflow from Natural Earth
  data to rnaturalearth enabling updating as new versions become
  available

- clarification of differences in world maps classified by countries,
  sovereign states and map units

- consistency with Natural Earth naming conventions so that
  rnaturalearth users can use Natural Earth documentation

- data in `sf` or `sv` formats

The [Natural Earth](https://www.naturalearthdata.com/) website
structures vector data by scale, category and type. These determine the
filenames of downloads. rnaturalearth uses this structure to facilitate
download (like an API).

## Install rnaturalearth

Install from CRAN :

``` r
install.packages("rnaturalearth")
```

or install the development version from GitHub using
[pak](https://github.com/r-lib/pak/).

``` r
pak::pkg_install("ropensci/rnaturalearth")
```

Data to support much of the package functionality are stored in two data
packages that you will be prompted to install when required if you do
not do so here.

``` r
pak::pkg_install("ropensci/rnaturalearthdata")
pak::pkg_install("ropensci/rnaturalearthhires")
```

## First usage

Here using `plot` as a simple, quick way to plot maps. Maps could also
be made with `ggplot2`, `tmap` or other options. All retrieval functions
accept an argument `returnclass = "sf"` to return package `sf` (Simple
Features) or `returnclass = "sv"` (SpatVector) objects.

``` r
library(rnaturalearth)

# world countries
plot(ne_countries())
Warning: plotting the first 9 out of 168 attributes; use max.plot = 168 to plot
all
```

![](man/figures/README-unnamed-chunk-2-1.png)<!-- -->

``` r

# uk
plot(ne_countries(country = "united kingdom"))
Warning: plotting the first 10 out of 168 attributes; use max.plot = 168 to
plot all
```

![](man/figures/README-unnamed-chunk-2-2.png)<!-- -->

``` r

# states, admin level1 boundaries
plot(ne_states(country = "spain"))
Warning: plotting the first 9 out of 121 attributes; use max.plot = 121 to plot
all
```

![](man/figures/README-unnamed-chunk-2-3.png)<!-- -->

## Introductory vignette

``` r
vignette("rnaturalearth", package = "rnaturalearth")
```

## To download Natural Earth data not already in the package

There are a wealth of other data available at the [Natural
Earth](https://www.naturalearthdata.com/) website. `rnaturalearth` has
functions to help with download of these data.

The data available are outlined in the two tables below and online
[here](https://www.naturalearthdata.com/downloads/50m-physical-vectors/).


    category   cultural 

    category   physical 

Specify the `scale`, `category` and `type` of the vector you want as in
the examples below.

``` r
# lakes
lakes110 <- ne_download(scale = 110, type = "lakes", category = "physical")
plot(lakes110)

# rivers
rivers50 <- ne_download(
  scale = 50,
  type = "rivers_lake_centerlines",
  category = "physical",
  returnclass = "sf"
)

library(ggplot2)
library(sf)

ggplot(rivers50) +
  geom_sf() +
  theme_minimal()
```

## Details of different country definitions and scales

``` r
vignette("what-is-a-country", package = "rnaturalearth")
```

## Reproducible download of Natural Earth data into the package

[Script](https://github.com/ropensci/rnaturalearthdata/blob/master/data-raw/data_download_script.R)
used to get data into the accompanying data packages.

## Errors in the data?

If you believe there is an issue with data provided by [Natural
Earth](https://www.naturalearthdata.com/), please do not report it here.
We are not responsible for the accuracy or maintenance of Natural Earth
data. For any concerns regarding this data, please contact [Natural
Earth](https://www.naturalearthdata.com/).

## Acknowledgements

Thanks to [Lincoln Mullen](https://github.com/lmullen) for code
structure inspiration from
[USAboundaries](https://github.com/ropensci/USAboundaries), [Hadley
Wickham](https://github.com/hadley) for comments and prompting, [Bob
Rudis](https://github.com/hrbrmstr) for answers to stackoverflow
questions about downloading Natural Earth data into R. The [Natural
Earth team](https://www.naturalearthdata.com/about/contributors/) and
[Nathan Kelso](https://github.com/nvkelso) for providing such a great
resource.

## Potential future work

### Potential additional data

- Country synonyms lookup
  - dataframe with ISO3 and country synonyms
  - similar to
    <https://github.com/AndySouth/rworldmap/blob/master/data/countrySynonyms.rda>
- Country larger regions lookup
  - dataframe with ISO3 and membership of different regional groupings,
    e.g. continent, least developed countries etc.
  - similar to
    <https://github.com/AndySouth/rworldmap/blob/master/data/countryRegions.rda>

### Potential additional functions

- facilitate joining of user data to country boundaries
  - similar to
    <https://github.com/AndySouth/rworldmap/blob/master/R/joinCountryData2Map.R>
  - … but with a better name
  - similar allowing of join by ISO codes or names, with attempted
    synonym matching
  - similar reporting of country joining success and failure
- facilitate subsetting by country groupings
  - e.g. least developed countries etc.

#' Check whether to install rnaturalearthdata and install if necessary
#'
#' If the rnaturalearthdata package is not installed, install it from GitHub
#' using pak. If it is not up to date, reinstall it.
#'
#' @export
check_rnaturalearthdata <- function() {
  rnaturalearthdata_version <- "0.0.0.9000"
  if (!requireNamespace("rnaturalearthdata", quietly = TRUE)) {
    cli::cli_inform(
      "The {.pkg rnaturalearthdata} package needs to be installed."
    )
    install_rnaturalearthdata()
  } else if (
    utils::packageVersion("rnaturalearthdata") < rnaturalearthdata_version
  ) {
    cli::cli_inform("The {.pkg rnaturalearthdata} package needs to be updated.")
    install_rnaturalearthdata()
  }
}

#' Install the naturalearthdata package after checking with the user
#' @export
install_rnaturalearthdata <- function() {
  error_func <- function(e) {
    cli::cli_abort(
      "Failed to install the {.pkg rnaturalearthdata} package.\nPlease try installing the package for yourself using the following command: {.code install.packages(\"rnaturalearthdata\")}"
    )
  }

  # 23/2/17 changed to try install if not interactive to avoid winbuilder
  # warning
  input <- 1L
  if (interactive()) {
    input <- utils::menu(
      c("Yes", "No"),
      title = "Install the rnaturalearthdata package?"
    )
  }

  if (input == 1L) {
    cli::cli_inform("Installing the {.pkg rnaturalearthdata} package.")
    tryCatch(
      utils::install.packages(
        "rnaturalearthdata",
        repos = c("http://packages.ropensci.org", "http://cran.rstudio.com"),
        type = "source"
      ),
      error = error_func,
      warning = error_func
    )
  } else {
    cli::cli_abort(
      "The {.pkg rnaturalearthdata} package is necessary for that method."
    )
  }
}

#' check whether the requested data exist on Natural Earth
#'
#' checks from a list dependent on type, category and scale. If it returns FALSE
#' the data may still exist on the website. Doesn't yet do checking on raster
#' names because I found the naming convention too tricky.
#'
#' @inherit ne_download
#'
#' @param type type of natural earth file to download one of 'countries',
#'    'map_units', 'map_subunits', 'sovereignty', 'states' OR the portion of any
#'    natural earth vector url after the scale and before the . e.g. for
#'    'ne_50m_urban_areas.zip' this would be 'urban_areas' OR the raster
#'    filename e.g. for 'MSR_50M.zip' this would be 'MSR_50M'
#'
#' @param category one of natural earth categories : 'cultural', 'physical',
#' 'raster'

#' @examples
#' check_data_exist(type = "countries", scale = 110, category = "cultural")
#'
#' # Type not in list for this category
#' check_data_exist(type = "airports", scale = 110, category = "physical")
#'
#' # Type in list but scale shows FALSE
#' check_data_exist(type = "airports", scale = 110, category = "cultural")
#'
#' @return TRUE or FALSE
#'
#' @export
check_data_exist <- function(
  type,
  scale = 110L,
  category = c("cultural", "physical", "raster")
) {
  # check permitted category
  category <- match.arg(category)

  # I would need to create a data_list_raster.csv file
  if (category == "raster") {
    return(TRUE)
  }

  # check on permitted scales, convert names to numeric
  scale <- check_scale(scale)

  df_data <- utils::read.csv(
    system.file(
      "extdata",
      paste0("data_list_", category, ".csv"),
      package = "rnaturalearth"
    )
  )

  # first check if type is within the list
  if (!type %in% df_data[["type"]]) {
    cli::cli_warn(
      "{.arg {type}} seems not to be in the list for category= {.val {category}} maybe try the other category of c('cultural', 'physical')"
    )

    return(FALSE)
  }

  exist <- df_data[df_data[["type"]] == type, paste0("scale", scale)]

  if (!exist) {
    cli::cli_warn(
      "The requested daa seem not to exist in the list of Natural Earth data. Check {.code ?ne_download} or {.url http://www.naturalearthdata.com/features/} to see data availability."
    )
  }

  return(exist)
}

#' check that this scale is present in Natural Earth
#'
#' check name or numeric scale representations, return numeric one
#'
#' @param x scale of map to return, one of \code{110}, \code{50}, \code{10} or
#' \code{'small'}, \code{'medium'}, \code{'large'}
#'
#' @return integer scale of map
check_scale <- function(x) {
  if (is.numeric(x) && length(x) == 1L) {
    if (x %in% c(110L, 50L, 10L)) {
      return(x)
    }
  } else if (is.character(x) && length(x) == 1L) {
    xnew <- c(small = 110L, medium = 50L, large = 10L)[tolower(x)]

    if (!is.na(xnew)) {
      return(unname(xnew))
    }
  }

  cli::cli_abort(
    "Invalid {.arg scale} argument. Must be one of {.val [110, 50, 10, 'small', 'medium', 'large']} you provided : {.val {x}}."
  )
}

#' Get natural earth world state (admin level 1) polygons
#'
#' returns state polygons (administrative level 1) for specified countries
#'
#' @inherit ne_download
#'
#' @param country a character vector of country names.
#' @param geounit a character vector of geounit names.
#' @param iso_a2 a character vector of iso_a2 country codes
#' @param spat_object an optional alternative states map
#'
#' @details
#' By default, this function uses the scale = 10 data from the
#' rnaturalearthhires package. While data at scales 50 and 110 exist,
#' they are not used as defaults because they contain fewer countries
#' (e.g. scale 50 only includes Australia, Brazil, Canada and United
#' States of America). For finer control, use \code{\link{ne_download}}.
#'
#' \code{ne_download( scale = 10L, type = "states", category = "cultural")}
#'
#' \code{ne_download( scale = 50L, type = "states", category = "cultural")}
#'
#' \code{ne_download( scale = 110L, type = "states", category = "cultural")}
#'
#' @aliases ne_admin1
#'
#' @examples
#'
#' # comparing using country and geounit to filter
#' if (requireNamespace("rnaturalearthhires")) {
#'   spdf_france_country <- ne_states(country = "france")
#'   spdf_france_geounit <- ne_states(geounit = "france")
#'
#'   plot(spdf_france_country)
#'   plot(spdf_france_geounit)
#'
#'   plot(ne_states(country = "united kingdom"))
#'   plot(ne_states(geounit = "england"))
#' }
#'
#' @export
ne_states <- function(
    country = NULL,
    geounit = NULL,
    iso_a2 = NULL,
    spat_object = NULL,
    returnclass = c("sf", "sv")
    ) {
  returnclass <- match.arg(returnclass)

  if (returnclass == "sp") {
    deprecate_sp("ne_download(returnclass = 'sp')")
  }

  # set map from one stored this adds potential to add or pass other potential
  # state maps, e.g. without lakes but no checking is done, may not be needed
  if (is.null(spat_object)) {
    check_rnaturalearthhires()
    spat_object <- rnaturalearthhires::states10
  }

  # set default filter
  filter <- TRUE

  # filter by country name (admin field in ne)
  if (!is.null(country)) {
    # check if field in data, for passed data or if changed in future
    if (!("admin" %in% names(spat_object))) {
      cli::cli_abort("No admin field in the data : {.val {names(spat_object)}}")
    }

    filter_country <- tolower(spat_object[["admin"]]) %in% tolower(country)
    filter <- filter & filter_country

    if (sum(filter_country) == 0L) {
      cli::cli_abort("No such country ({.val {country}}) in the data.")
    }
  }

  # filter by geounit
  if (!is.null(geounit)) {
    if (!("geonunit" %in% names(spat_object))) {
      cli::cli_abort(
        "No geonunit field in the data : {names(spat_object)}"
      )
    }

    filter_geounit <- tolower(spat_object[["geonunit"]]) %in% tolower(geounit)
    filter <- filter & filter_geounit

    if (sum(filter_geounit) == 0L) {
      cli::cli_abort("No such geounit ({.val {geounit}}) in the data.")
    }
  }

  # filter by iso_a2
  if (!is.null(iso_a2)) {
    if (!("iso_a2" %in% names(spat_object))) {
      cli::cli_abort(
        "No {.var iso_a2} field in the data : {names(spat_object)}."
      )
    }

    filter_iso_a2 <- tolower(spat_object[["iso_a2"]]) %in% tolower(iso_a2)
    filter <- filter & filter_iso_a2

    if (sum(filter_iso_a2) == 0L) {
      cli::cli_abort("No such iso_a2 ({.val {iso_a2}}) in the data.")
    }
  }

  # Convert to the desired class
  convert_spatial_class(spat_object[filter, ], returnclass)
}

# TODO: Change to sf in rnaturalearthhires
test_that("type of object returned is as expected", {
  skip_on_cran()
  skip_if_not_installed("rnaturalearthdata")
  skip_if_not_installed("rnaturalearthhires")

  expect_is(ne_states(country = "france"), "sf")
  expect_is(ne_states(geounit = "france"), "sf")
  expect_is(ne_states(country = "united kingdom"), "sf")
  expect_is(ne_states(geounit = "england"), "sf")
})

test_that("filter by country gives bigger object than by geounit", {
  skip_on_cran()
  skip_if_not_installed("rnaturalearthdata")
  skip_if_not_installed("rnaturalearthhires")

  expect_gt(
    object.size(ne_states(country = "france")),
    object.size(ne_states(geounit = "france"))
  )

  expect_gt(
    object.size(ne_states(country = "united kingdom")),
    object.size(ne_states(geounit = "england"))
  )
})

test_that("equivalent country and iso_a2 arguments give identical results", {
  skip_on_cran()
  skip_if_not_installed("rnaturalearthdata")
  skip_if_not_installed("rnaturalearthhires")

  expect_identical(
    ne_states(country = "france"),
    ne_states(iso_a2 = "FR")
  )

  expect_identical(
    ne_states(country = "united kingdom"),
    ne_states(iso_a2 = "GB")
  )
})

test_that("Error message if incorrect filters are applied", {
  skip_on_cran()
  skip_if_not_installed("rnaturalearthdata")
  skip_if_not_installed("rnaturalearthhires")

  expect_error(ne_states(country = "madeupname"))
  expect_error(ne_states(geounit = "madeupname"))
  expect_error(ne_states(iso_a2 = "madeupname"))
})

test_that("type of object returned is sf", {
  skip_on_cran()
  skip_if_not_installed("rnaturalearthdata")
  skip_if_not_installed("rnaturalearthhires")

  world <- ne_countries()
  africa <- ne_countries(continent = "africa")
  france <- ne_countries(country = "france")

  expect_is(world, "sf")
  expect_is(africa, "sf")
  expect_is(france, "sf")
})

test_that("type of object returned is SpatVector", {
  skip_on_cran()
  skip_if_not_installed("rnaturalearthdata")
  skip_if_not_installed("rnaturalearthhires")

  world <- ne_countries(returnclass = "sv")
  africa <- ne_countries(continent = "africa", returnclass = "sv")
  france <- ne_countries(country = "france", returnclass = "sv")

  expect_is(world, "SpatVector")
  expect_is(africa, "SpatVector")
  expect_is(france, "SpatVector")
})

# Check if I should do all the tests for all the scales.
test_that("type of object returned is either sf or SpatVector for coastlines", {
  skip_on_cran()
  skip_if_not_installed("rnaturalearthdata")
  skip_if_not_installed("rnaturalearthhires")

  expect_true(inherits(ne_coastline(returnclass = "sf", scale = 10), "sf"))
  expect_true(inherits(ne_coastline(returnclass = "sf", scale = 50), "sf"))
  expect_true(inherits(ne_coastline(returnclass = "sf", scale = 110), "sf"))
  expect_true(inherits(ne_coastline(returnclass = "sv"), "SpatVector"))
})

test_that("type of object returned is either sf or SpatVector for states", {
  skip_on_cran()
  skip_if_not_installed("rnaturalearthdata")
  skip_if_not_installed("rnaturalearthhires")

  expect_true(inherits(ne_states(returnclass = "sf"), "sf"))
  expect_true(inherits(ne_states(returnclass = "sv"), "SpatVector"))
})

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom sf as_Spatial
#' @importFrom terra vect
#' @importFrom utils read.csv
#' @import httr
#' @importFrom jsonlite fromJSON
## usethis namespace: end
NULL

#' Check whether to install rnaturalearthhires and install if necessary
#'
#' If the rnaturalearthhires package is not installed, install it from GitHub
#' using pak. If it is not up to date, reinstall it.
#'
#' @export
check_rnaturalearthhires <- function() {
  # TODO: This should be done dynamically by checking the version on Github
  rnaturalearthhires_version <- "1.0.0.9000"

  if (!requireNamespace("rnaturalearthhires", quietly = TRUE)) {
    cli::cli_inform(
      "The {.pkg rnaturalearthhires} package needs to be installed."
    )
    install_rnaturalearthhires()
  } else if (
    utils::packageVersion("rnaturalearthhires") < rnaturalearthhires_version
  ) {
    cli::cli_inform(
      "The {.pkg rnaturalearthhires} package needs to be updated."
    )
    install_rnaturalearthhires()
  }
}

#' Install the naturalearthhires package after checking with the user
#' @export
install_rnaturalearthhires <- function() {
  input <- 1L

  if (interactive()) {
    input <- utils::menu(
      c("Yes", "No"),
      title = "Install the rnaturalearthhires package?"
    )
  }

  if (input != 1L) {
    cli::cli_abort(
      "The {.pkg rnaturalearthhires} package is necessary for that method.\n Please try installing the package yourself with {.code pak::pkg_install(\"ropensci/rnaturalearthhires\")}"
    )
  }

  cli::cli_inform("Installing the {.pkg rnaturalearthhires} package.")

  tryCatch(
    pak::pkg_install("ropensci/rnaturalearthhires"),
    error = function(e) {
      cli::cli_inform(conditionMessage(e))
    }
  )
}

#' return a natural earth filename based on arguments
#'
#' returns a string that can then be used to download the file.
#'
#' @inherit ne_download
#'
#' @param type type of natural earth file to download one of 'countries',
#'    'map_units', 'map_subunits', 'sovereignty', 'states' OR the portion of any
#'    natural earth vector url after the scale and before the . e.g. for
#'    'ne_50m_urban_areas.zip' this would be 'urban_areas' OR the raster
#'    filename e.g. for 'MSR_50M.zip' this would be 'MSR_50M'
#'
#' @param category one of natural earth categories : 'cultural', 'physical',
#' 'raster'
#'
#' @examples
#' ne_url <- ne_file_name(scale = 110, type = "countries")
#'
#' @return string
#'
#' @export
ne_file_name <- function(
  scale = 110L,
  type = "countries",
  category = c("cultural", "physical", "raster")
) {
  scale <- check_scale(scale)
  category <- match.arg(category)

  type <- normalize_type(type)

  base_url <- "/vsizip//vsicurl/https://naciscdn.org/naturalearth"

  # Construct the file path based on the data category
  gdal_url <- if (category == "raster") {
    file.path(
      base_url,
      sprintf("%sm", scale),
      category,
      sprintf("%s.zip", type),
      sprintf("%s.tif", type)
    )
  } else {
    file.path(
      base_url,
      sprintf("%sm", scale),
      category,
      sprintf("ne_%sm_%s.zip", scale, type)
    )
  }

  return(gdal_url)
}

#' Normalize the type argument for Natural Earth datasets
#'
#' This function standardizes the `type` argument by mapping common names to
#' their respective Natural Earth dataset names.
#'
#' @inheritParams ne_file_name
#'
#' @return A string representing the normalized dataset type.
normalize_type <- function(type) {
  if (
    type %in%
      c(
        "countries",
        "map_units",
        "map_subunits",
        "sovereignty",
        "tiny_countries",
        "boundary_lines_land",
        "pacific_groupings",
        "breakaway_disputed_areas",
        "boundary_lines_disputed_areas",
        "boundary_lines_maritime_indicator"
      )
  ) {
    return(paste0("admin_0_", type))
  }

  if (
    type %in%
      c(
        "parks_and_protected_lands_area",
        "parks_and_protected_lands_line",
        "parks_and_protected_lands_point",
        "parks_and_protected_lands_scale_rank"
      )
  ) {
    return("parks_and_protected_lands")
  }

  if (type == "states") {
    return("admin_1_states_provinces_lakes")
  }

  type
}

#' Generate the layer name for a Natural Earth dataset
#'
#' @inheritParams ne_file_name
#' @return A string representing the dataset layer name.
layer_name <- function(type, scale) {
  if (
    type %in%
      c(
        "countries",
        "map_units",
        "map_subunits",
        "sovereignty",
        "tiny_countries",
        "boundary_lines_land",
        "pacific_groupings",
        "breakaway_disputed_areas",
        "boundary_lines_disputed_areas",
        "boundary_lines_maritime_indicator"
      )
  ) {
    type <- paste0("admin_0_", type)
  }

  if (type == "states") {
    type <- "admin_1_states_provinces_lakes"
  }

  sprintf("ne_%sm_%s", scale, type)
}

library(httr)

url_exists <- function(url) {
  url <- sanitize_gdal_url(url)
  res <- HEAD(url)
  status_code(res) == 200L
}

# a non exhaustive list of ne download urls
# initially taken from those in data_download_script.r
# fmt: skip
urls <- c(
  ne_file_name(scale = 110L, type = "countries", category = "cultural"),
  ne_file_name(scale = 110L, type = "map_units", category = "cultural"),
  ne_file_name(scale = 110L, type = "sovereignty", category = "cultural"),
  ne_file_name(scale = 50L, type = "countries", category = "cultural"),
  ne_file_name(scale = 50L, type = "map_units", category = "cultural"),
  ne_file_name(scale = 50L, type = "sovereignty", category = "cultural"),
  ne_file_name(scale = 10L, type = "countries", category = "cultural"),
  ne_file_name(scale = 10L, type = "map_units", category = "cultural"),
  ne_file_name(scale = 10L, type = "sovereignty", category = "cultural"),
  ne_file_name(scale = 50L, type = "states", category = "cultural"),
  ne_file_name(scale = 10L, type = "states", category = "cultural"),
  ne_file_name(scale = 110L, type = "tiny_countries", category = "cultural"),
  ne_file_name(scale = 110L, type = "coastline", category = "physical"),
  ne_file_name(scale = 50L, type = "coastline", category = "physical"),
  ne_file_name(scale = 10L, type = "coastline", category = "physical")
)

url_expect_fail <- ne_file_name(
  scale = 110L,
  type = "expect_fail",
  category = "cultural"
)

test_that("urls for downloads created by the package exist", {
  skip_on_cran()
  skip_if_not_installed("rnaturalearthdata")
  skip_if_not_installed("rnaturalearthhires")

  expect_true(all(vapply(urls, url_exists, logical(1L))))
})

test_that("a bogus url does not exist", {
  skip_on_cran()
  skip_if_not_installed("rnaturalearthdata")
  skip_if_not_installed("rnaturalearthhires")

  expect_false(url_exists(url_expect_fail))
})

# Use this to create the vignette to prevent CRAN checks from failing when
# internet is not available

# https://ropensci.org/blog/2019/12/08/precompute-vignettes/

library(fs)

infiles <- dir_ls("vignettes", glob = "*.orig")
outfiles <- fs::path_ext_remove(infiles)

purrr::walk2(infiles, outfiles, knitr::knit)

figfiles <- fs::dir_ls(glob = "*.png")

fs::file_move(figfiles, "vignettes")

#' Return a dataframe of available vector layers on Natural Earth
#'
#' Checks the Natural Earth Github repository for current vector layers and
#' provides the file name required in the type argument of ne_download.
#'
#' @inherit ne_download
#'
#' @param category one of natural earth categories : 'cultural', 'physical'
#'
#' @param getmeta whether to get url of the metadata for each layer
#'
#' @return dataframe with two variables: layer and metadata
#'
#' @export
#'
#' @examples \dontrun{
#' ne_find_vector_data(scale = 10, category = "physical")
#' }
ne_find_vector_data <- function(
  scale = 110L,
  category = c("cultural", "physical"),
  getmeta = FALSE
) {
  ## check permitted category (no way to check against available rasters)
  category <- match.arg(category)

  ## check on permitted scales, convert names to numeric
  scale <- check_scale(scale)

  ## Available paths include: 10m_cultural, 10m_physical,
  ## 50m_cultural, etc...

  # scale has already been checked in check_scale, and category in match.arg
  path <- paste0(scale, "m_", category)

  ## call to ne_git_contents returns a list with contents of
  ## github directory (based on specified path), github api
  ## response, and http path.

  resp <- ne_git_contents(path = path)

  ## call to git layer names returns a list of lists with
  ## valid layer names and metadata link

  layers <- ne_git_layer_names(x = resp, scale = scale, getmeta = getmeta)

  ## I think returning as a data.frame makes the most sense

  if (getmeta) {
    layers <- data.frame(
      layer = layers[["layer"]],
      scale = scale,
      metadata = layers[["metalink"]]
    )
  } else {
    layers <- data.frame(
      layer = layers[["layer"]],
      scale = scale
    )
  }

  return(layers)
}

#' Return contents of Natural Earth Github directory
#'
#' Uses the Github API to return contents of Natural Earth Github directories.
#' @param path string, one of: \code{'110m_physical'}, \code{'110m_cultural'},
#'   \code{'50m_physical'}, \code{'50m_cultural'}, \code{'10m_physical'},
#'   \code{'10m_cultural'}
#'
#' @return list. Includes parsed json content, http path, and response
#'   code.
#'
#' @keywords internal
ne_git_contents <- function(path) {
  ## create pathnames to natural earth vector folders on github use httr::GET
  ## and the github api to access the github contents API. Note that this is
  ## rate limited, so someone could get locked. Probably don't want to include
  ## this in CRAN tests.

  path <- paste0("/repos/nvkelso/natural-earth-vector/contents/", path)
  url <- httr::modify_url("https://api.github.com", path = path)
  ua <- httr::user_agent("http://github.com/ropensci/rnaturalearth")
  resp <- httr::GET(url, ua)

  if (httr::http_type(resp) != "application/json") {
    cli::cli_abort("API did not return json")
  }

  df <- httr::content(x = resp, as = "text", encoding = "UTF-8")
  df <- jsonlite::fromJSON(df)

  if (httr::status_code(resp) != 200L) {
    cli::cli_abort(
      sprintf(
        "GitHub API request failed [%s]\n%s\n<%s>",
        httr::status_code(resp),
        df[["message"]],
        df[["documentation_url"]]
      )
    )
  }

  structure(
    list(
      content = df,
      path = path,
      response = resp
    )
  )
}


#' Create list of layer names and metadata links
#'
#' Parses Natural Earth Github folder content for layer names and metadata
#' links.
#'
#' @param x object returned by ne_git_contents
#'
#' @param scale one of \code{110}, \code{50}, \code{10}
#'
#' @param getmeta whether to get url of the metadata for each layer
#'
#' @return list of lists with layer names and metadata links.

#' @keywords internal
ne_git_layer_names <- function(x, scale, getmeta) {
  ## uses the output from ne_git_contents
  ## creates a list of available layer names and
  ## list of metadata links

  if (httr::status_code(x$response) != 200L) {
    cli::cli_abort(
      sprintf(
        "GitHub API request failed [%s]\n%s\n<%s>",
        httr::status_code(x$response),
        x$content$message,
        x$content$documentation_url
      )
    )
  }

  ## Create the pattern that matches the prefix that should be removed
  prefix <- paste0("ne_", scale, "m_")

  ## clean and return layer names
  l <- x$content
  # gets just readme because 1 per layer
  l <- regmatches(l$name, regexpr("^(.*).README.html", l$name))
  # cuts off parts of filenames to get just the layer name
  l <- gsub(".README.html", "", l)
  l <- gsub(prefix, "", l)

  # if we didn't need links to metadata then this function could stop here
  if (!getmeta) {
    ## return a list with just layer name
    return(structure(list(layer = l)))
  }

  ## clean and return links to the metadata on NaturalEarthData.com
  m <- x$content$download_url
  m <- regmatches(m, regexpr("^(.*).README.html", m))

  findlinks <- function(x) {
    page <- httr::content(httr::GET(x))
    link <- regmatches(
      page,
      regexpr(
        '<link rel="canonical" href=(.*?)/>',
        page
      )
    )
    link <- gsub("<link rel=\"canonical\" href=\"", "", link)
    link <- gsub("\" />", "", link)
  }

  ## iterate findlinks function through each html page to
  ## pull out the metadata link. Adding an optional status bar
  ## since downloads could stall.
  ## pbapply is suggested,
  ## if user doesn't have pbapply installed, lapply is used.
  m <- if (requireNamespace("pbapply", quietly = TRUE)) {
    pbapply::pblapply(m, findlinks)
  } else {
    lapply(m, findlinks)
  }

  m <- unlist(m)

  ## return a list with layer name and metadata link
  structure(list(layer = l, metalink = m))
}

#' Get natural earth world country polygons
#'
#' returns world country polygons at a specified scale, or points of
#' tiny_countries
#'
#' @inherit ne_download
#'
#' @param type country type, one of 'countries', 'map_units', 'sovereignty',
#' 'tiny_countries'
#'
#' @param continent a character vector of continent names to get countries from.
#'
#' @param country a character vector of country names.
#'
#' @param geounit a character vector of geounit names.
#'
#' @param sovereignty a character vector of sovereignty names.
#'
#' @aliases ne_admin0
#'
#' @examples
#' world <- ne_countries()
#' africa <- ne_countries(continent = "africa")
#' france <- ne_countries(country = "france")
#'
#' plot(world$geometry)
#' plot(africa$geometry)
#' plot(france$geometry)
#'
#' # get as SpatVector
#' world <- ne_countries(returnclass = "sv")
#' terra::plot(world)
#'
#' tiny_countries <- ne_countries(type = "tiny_countries", scale = 50)
#' plot(tiny_countries)
#'
#' @export
ne_countries <- function(
    scale = 110L,
    type = "countries",
    continent = NULL,
    country = NULL,
    geounit = NULL,
    sovereignty = NULL,
    returnclass = c("sf", "sv")
    ) {
  returnclass <- match.arg(returnclass)

  if (returnclass == "sp") {
    deprecate_sp("ne_download(returnclass = 'sp')")
  }

  spat_object <- get_data(scale = scale, type = type)

  # some large scale NE data still have old uppercase fieldnames, this to
  # correct
  names(spat_object) <- tolower(names(spat_object))

  # set default filter
  filter <- TRUE

  # filter by continent
  if (!is.null(continent)) {
    filter <- tolower(spat_object[["continent"]]) %in% tolower(continent)

    if (sum(filter) == 0L) {
      cli::cli_abort("No such continent ({.val {continent}}) in the data.")
    }
  }

  # filter by country name (admin field in ne) todo I might be able to add the
  # name field from ne in here too
  if (!is.null(country)) {
    filter_country <- tolower(spat_object[["admin"]]) %in% tolower(country)
    filter <- filter & filter_country

    if (sum(filter_country) == 0L) {
      cli::cli_abort("No such country ({.val {country}}) in the data.")
    }
  }

  # filter by geounit
  if (!is.null(geounit)) {
    filter_geounit <- tolower(spat_object[["geounit"]]) %in% tolower(geounit)
    filter <- filter & filter_geounit

    if (sum(filter_geounit) == 0L) {
      cli::cli_abort("No such geounit ({.val {geounit}}) in the data.")
    }
  }

  # filter by sovereignty (BEWARE its called sovereignt in ne)
  if (!is.null(sovereignty)) {
    filter_sovereignty <- tolower(spat_object[["sovereignt"]]) %in%
      tolower(sovereignty)
    filter <- filter & filter_sovereignty

    if (sum(filter_sovereignty) == 0L) {
      cli::cli_abort("No such sovereignty ({.val {sovereignty}}) in the data")
    }
  }

  # Convert to the desired class
  convert_spatial_class(spat_object[filter, ], returnclass)
}

test_that("scale argument gives expected relative sizes of objects", {
  skip_on_cran()
  skip_if_not_installed("rnaturalearthdata")
  skip_if_not_installed("rnaturalearthhires")

  expect_gt(
    object.size(ne_countries(scale = "medium")),
    object.size(ne_countries(scale = "small"))
  )

  expect_gt(
    object.size(ne_countries(scale = 50)),
    object.size(ne_countries(scale = 110))
  )
})

test_that("equivalent scale arguments as numeric or text give identical results", {
  skip_on_cran()
  skip_if_not_installed("rnaturalearthdata")
  skip_if_not_installed("rnaturalearthhires")

  expect_identical(
    ne_countries(scale = "medium"),
    ne_countries(scale = 50)
  )

  expect_identical(
    ne_countries(scale = "small"),
    ne_countries(scale = 110)
  )
})

test_that("Error message if incorrect filters are applied", {
  skip_on_cran()
  skip_if_not_installed("rnaturalearthdata")
  skip_if_not_installed("rnaturalearthhires")

  expect_error(ne_countries(continent = "madeupname"))
  expect_error(ne_countries(country = "madeupname"))
  expect_error(ne_countries(geounit = "madeupname"))
  expect_error(ne_countries(sovereignty = "madeupname"))
})

test_that("Error message if incorrect type argument", {
  skip_on_cran()
  skip_if_not_installed("rnaturalearthdata")
  skip_if_not_installed("rnaturalearthhires")

  expect_error(ne_countries(type = "madeupname"))
})

#' world country polygons from Natural Earth
#'
#' at 1:110m scale (small). Other data and resolutions are in the packages
#' rnaturalearthdata and rnaturalearthhires.
#'
#' @format A \code{sf} object.
#'
#' @slot data A data frame with country attributes.
#'
#' @aliases countries110
#'
#' @name countries
NULL
#'
#' @source
#' \url{https://naciscdn.org/naturalearth/10m/cultural/ne_10m_admin_0_countries.zip}
#'
#' @rdname countries
"countries110"
#'
#' list of cultural layers available from Natural Earth
#'
#' @format A \code{DataFrame}
#' @aliases df_layers_cultural
#' @name df_layers_cultural
#'
NULL

#' @rdname df_layers_cultural
"df_layers_cultural"

#' list of physical layers available from Natural Earth
#'
#'
#' @format A \code{DataFrame}
#' @aliases df_layers_physical
#' @name df_layers_physical
#'
NULL

#' @rdname df_layers_physical
"df_layers_physical"

#' Read Spatial Vector
#'
#' This function reads a spatial vector file and returns either an `sf` object
#' or a `sv` object.
#'
#' @param x A character string specifying the path to the spatial vector file.
#' @param returnclass A character string specifying the class of object to
#' return. Options are "sf" for sf object and "sv" for sv object.
#' @param layer A character string specifying the later to read in case there
#' are more than one in the zip file.
#'
#' @return Either an `sf` object or a `sv` object.
read_spatial_vector <- function(x, layer, returnclass = c("sf", "sv")) {
  returnclass <- match.arg(returnclass)
  switch(
    returnclass,
    sf = sf::st_read(x, layer = layer, quiet = TRUE, stringsAsFactors = FALSE),
    sv = terra::vect(x, layer = layer)
  )
}

#' Convert from/to sf/sv objects
#'
#' @inherit ne_download
#' @param x Object to be converted
#' @return Object of class "sf" or "sv"
convert_spatial_class <- function(x, returnclass = c("sf", "sv")) {
  returnclass <- match.arg(returnclass)
  switch(returnclass, sf = sf::st_as_sf(x), sv = terra::vect(x))
}

#' @title Extracts the http URL from a VSIZIP URL
#' @description This function takes a VSIZIP URL and extracts the http URL from
#' it
#' @param url A character string representing the VSIZIP URL
#' @return A character string representing the http URL extracted from the
#' VSIZIP URL
sanitize_gdal_url <- function(url) {
  url <- sub("^/vsizip//vsicurl/", "", url)
  sub("(.+\\.zip)/.*", "\\1", url)
}

#' Create Destination File Path
#'
#' @description Creates a destination file path by combining the directory path
#' with a formatted filename based on the GDAL URL and category.
#'
#' @param gdal_url A character string representing the GDAL URL
#' @param category A character string specifying the data category ("raster" or other)
#' @param destdir A character string specifying the destination directory
#'
#' @return A character string representing the complete destination file path
make_dest_path <- function(gdal_url, category, destdir) {
  file.path(
    destdir,
    sprintf(
      "%s.%s",
      tools::file_path_sans_ext(basename(gdal_url)),
      ifelse(category == "raster", "tif", "gpkg")
    )
  )
}

#' load a Natural Earth vector that has already been downloaded to R using
#' \code{\link{ne_download}}
#'
#' returns loaded data as a spatial object.
#'
#' @inherit ne_download
#'
#' @param type type of natural earth file one of 'countries', 'map_units',
#'    'map_subunits', 'sovereignty', 'states' OR the portion of any natural
#'    earth vector url after the scale and before the . e.g. for
#'    'ne_50m_urban_areas.zip' this would be 'urban_areas' OR the raster
#'    filename e.g. for 'MSR_50M.zip' this would be 'MSR_50M'
#'
#' @param category one of natural earth categories : 'cultural', 'physical',
#' 'raster'
#'
#' @param destdir folder to load files from, default = tempdir()
#'
#' @param file_name OPTIONAL name of file (excluding path) instead of natural
#' earth attributes
#'
#' @details This function should be used after first downloading the data with
#' \code{ne_download(load = FALSE)}. The downloaded file can then be loaded
#' using this function.
#'
#' @seealso \code{\link{ne_download}}
#'
#' @examples \dontrun{
#' # download followed by load from tempdir() works in same R session
#' spdf_world <- ne_download(
#'   scale = 110,
#'   type = "countries"
#' )
#' spdf_world2 <- ne_load(
#'   scale = 110,
#'   type = "countries"
#' )
#'
#' # download followed by load from specified directory works between R sessions
#' spdf_world <- ne_download(
#'   scale = 110,
#'   type = "countries",
#'   destdir = getwd()
#' )
#' spdf_world2 <- ne_load(
#'   scale = 110,
#'   type = "countries",
#'   destdir = getwd()
#' )
#'
#' # for raster download & load
#' rst <- ne_download(
#'   scale = 50,
#'   type = "OB_50M",
#'   category = "raster",
#'   destdir = getwd(),
#'   load = FALSE
#' )
#'
#' # load after having downloaded
#' rst <- ne_load(
#'   scale = 50,
#'   type = "OB_50M",
#'   category = "raster",
#'   destdir = getwd()
#' )
#'
#' # plot
#' library(terra)
#' plot(rst)
#' # end dontrun
#' }
#'
#' @export
ne_load <- function(
    scale = 110L,
    type = "countries",
    category = c("cultural", "physical", "raster"),
    destdir = tempdir(),
    file_name = NULL,
    returnclass = c("sf", "sv")
    ) {
  category <- match.arg(category)

  returnclass <- match.arg(returnclass)
  scale <- check_scale(scale)

  if (returnclass == "sp") {
    deprecate_sp("ne_download(returnclass = 'sp')")
  }

  if (is.null(file_name)) {
    file_name <- ne_file_name(scale = scale, type = type, category = category)
  }

  spatial_file_path <- make_dest_path(file_name, category, destdir)

  error_msg <- "The file {.path {spatial_file_path}} seems not to exist in your local folder {.path {destdir}}. Did you download it using {.fn rnaturalearth::ne_download}?"

  if (!file.exists(spatial_file_path)) {
    cli::cli_abort(error_msg)
  }

  if (category == "raster") {
    rst <- terra::rast(spatial_file_path)

    return(rst)
  } else {
    layer <- layer_name(type, scale)

    # read in data as either sf of spatvector
    spatial_object <- read_spatial_vector(
      spatial_file_path,
      layer = layer,
      returnclass
    )

    return(spatial_object)
  }
}

test_that("ne_load works for raster files", {
  skip_on_cran()
  skip_if_not_installed("rnaturalearthdata")
  skip_if_not_installed("rnaturalearthhires")

  scale <- 50L
  type <- "OB_50M"
  category <- "raster"
  tmpdir <- tempdir()

  ne_download(
    scale = scale,
    type = type,
    category = category,
    destdir = tmpdir,
    load = FALSE
  )

  rst <- ne_load(
    scale = scale,
    type = type,
    category = category,
    destdir = tmpdir
  )

  expect_s4_class(rst, "SpatRaster")
})

test_that("ne_load works for vector files", {
  skip_on_cran()
  skip_if_not_installed("rnaturalearthdata")
  skip_if_not_installed("rnaturalearthhires")

  tmpdir <- tempdir()

  scale_small <- 50L

  ne_download(
    scale = scale_small,
    destdir = tmpdir,
    load = FALSE
  )

  vect <- ne_load(scale = scale_small, destdir = tmpdir, returnclass = "sv")
  expect_s4_class(vect, "SpatVector")

  vect <- ne_load(scale = scale_small, destdir = tmpdir, returnclass = "sf")
  expect_s3_class(vect, "sf")

  scale_medium <- "medium"

  ne_download(
    scale = scale_medium,
    type = "land",
    category = "physical",
    destdir = tmpdir,
    load = FALSE
  )

  vect <- ne_load(
    scale = scale_medium,
    type = "land",
    category = "physical",
    destdir = tmpdir
  )

  expect_s3_class(vect, "sf")
})

deprecate_sp <- function(what, env = rlang::caller_env()) {
  lifecycle::deprecate_stop(
    when = "1.0.0",
    what = what,
    details = "Please use `sf` objects with {rnaturalearth}, Spatial objects (sp) are no longer supported. For example: `ne_download(returnclass = 'sf')`",
    env = env
  )
}

#' Get data from within the package
#'
#' returns world country polygons at a specified scale, used by ne_countries()
#'
#' @inherit ne_download
#'
#' @param type country type, one of 'countries', 'map_units', 'sovereignty',
#' 'tiny_countries'
#'
#' @return A \code{sf} object.
get_data <- function(
  scale = 110L,
  type = c(
    "countries",
    "map_units",
    "sovereignty",
    "tiny_countries"
  )
) {
  # check on permitted scale arg, convert names to numeric
  scale <- check_scale(scale)
  # check permitted type arg
  type <- match.arg(type)

  # tiny_countries not available at scale 10
  if (type == "tiny_countries" && scale == 10L) {
    cli::cli_abort(
      "{.arg tiny_countries} are not available at scale 10, use scale 50 or 110."
    )
  }

  # check for the data packages and try to install if not there
  # avoid this check for one example dataset in this package (i.e. countries110)

  if (scale == 10L) {
    check_rnaturalearthhires()
  } else if (!(scale == 110L && type == "countries")) {
    check_rnaturalearthdata()
  }

  # choose which map based on type and scale (stored in different packages)

  if (scale == 110L && type == "countries") {
    sf_object <- getExportedValue("rnaturalearth", paste0(type, scale))
  } else if (scale == 110L || scale == 50L) {
    sf_object <- getExportedValue("rnaturalearthdata", paste0(type, scale))
  } else if (scale == 10L) {
    sf_object <- getExportedValue("rnaturalearthhires", paste0(type, scale))
  }

  return(sf_object)
}

#' Download data from Natural Earth and (optionally) read into R
#'
#' returns downloaded data as a spatial object or the filename if
#' \code{load=FALSE}. if \code{destdir} is specified the data can be reloaded in
#' a later R session using \code{\link{ne_load}} with the same arguments.
#'
#' @param scale The scale of map to return, one of `110`, `50`, `10` or `small`,
#' `medium`, `large`.
#'
#' @param type type of natural earth file to download one of 'countries',
#' 'map_units', 'map_subunits', 'sovereignty', 'states' OR the portion of any
#' natural earth vector url after the scale and before the . e.g. for
#' 'ne_50m_urban_areas.zip' this would be 'urban_areas'. See Details. OR the
#' raster filename e.g. for 'MSR_50M.zip' this would be 'MSR_50M'
#'
#' @param category one of natural earth categories : 'cultural', 'physical',
#' 'raster'
#'
#' @param destdir where to save files, defaults to \code{tempdir()},
#' \code{getwd()} is also possible.
#'
#' @param load `TRUE` to load the spatial object into R, `FALSE` to return the
#' filename of the downloaded object. If the requested object is a vector, it
#' will be saved as a GPKG file. If a raster is requested, it will be saved as
#' a GeoTIFF file.
#'
#' @details Note that the filename of the requested object will be returned if
#' `load = FALSE`.
#'
#' @details
#' **Caching behavior:** When `load = TRUE` and `destdir` is set to a persistent
#' directory (not `tempdir()`), the function will check if the file already
#' exists locally. If it does, it will load from the cached file instead of
#' re-downloading. If the file doesn't exist, it will download from Natural Earth,
#' save a copy to `destdir`, and then load it into R. This allows you to use the
#' same code whether it's the first download or a subsequent load, avoiding
#' redundant downloads.
#'
#' @details
#' If `load = TRUE` and `destdir = tempdir()` (the default), the download will
#' be handled using the GDAL virtual file system, allowing direct access to the
#' data without writing it to disk.
#'
#' @seealso \code{\link{ne_load}}, pre-downloaded data are available using
#'   \code{\link{ne_countries}}, \code{\link{ne_states}}. Other geographic data
#'   are available in the raster package : \code{\link[raster]{getData}}.
#'
#' @param returnclass A string determining the spatial object to return. Either
#' "sf" for for simple feature (from `sf`, the default) or "sv" for a
#' `SpatVector` (from `terra`).
#'
#' @return An object of class `sf` for simple feature (from `sf`, the default)
#' or `SpatVector` (from `terra`).
#'
#' @examples \dontrun{
#' # Download and load into R (default behavior, no local copy saved)
#' spdf_world <- ne_download(scale = 110, type = "countries")
#' plot(spdf_world)
#'
#' # Download with caching: first call downloads and saves, subsequent calls load from cache
#' # First call: downloads from Natural Earth and saves to "my_maps" directory
#' spdf_world <- ne_download(scale = 110, type = "countries", destdir = "my_maps")
#'
#' # Second call: loads from cached file in "my_maps" (no re-download)
#' spdf_world2 <- ne_download(scale = 110, type = "countries", destdir = "my_maps")
#'
#' # This works the same across R sessions - same code, caches automatically
#'
#' # You can also use ne_load() to explicitly load from cache
#' spdf_world3 <- ne_load(scale = 110, type = "countries", destdir = "my_maps")
#'
#' # Download and save without loading into R
#' file_path <- ne_download(
#'   scale = 110, type = "countries",
#'   destdir = "my_maps", load = FALSE
#' )
#'
#' # Raster example with caching
#' rst <- ne_download(
#'   scale = 50, type = "MSR_50M",
#'   category = "raster", destdir = "my_maps"
#' )
#' # Subsequent calls will use cached version
#' rst2 <- ne_download(
#'   scale = 50, type = "MSR_50M",
#'   category = "raster", destdir = "my_maps"
#' )
#'
#' library(terra)
#' terra::plot(rst)
#' # end dontrun
#' }
#'
#' @export
ne_download <- function(
  scale = 110L,
  type = "countries",
  category = c("cultural", "physical", "raster"),
  destdir = tempdir(),
  load = TRUE,
  returnclass = c("sf", "sv")
) {
  category <- match.arg(category)
  returnclass <- match.arg(returnclass)

  if (!dir.exists(destdir)) {
    cli::cli_abort("{.arg destdir} must be an existing directory")
  }

  if (returnclass == "sp") {
    deprecate_sp("ne_download(returnclass = 'sp')")
  }

  scale <- check_scale(scale)

  gdal_url <- ne_file_name(
    scale = scale,
    type = type,
    category = category
  )

  # Check if we should use cached file
  # When load = TRUE and destdir is not tempdir, use caching behavior
  use_cache <- load && !identical(destdir, tempdir())

  if (use_cache) {
    spatial_file_path <- make_dest_path(gdal_url, category, destdir)

    # If file exists in cache, load it
    if (file.exists(spatial_file_path)) {
      cli::cli_inform(
        "Loading {.file {basename(spatial_file_path)}} from cache..."
      )

      if (category == "raster") {
        return(terra::rast(spatial_file_path))
      } else {
        layer <- layer_name(type, scale)
        return(read_spatial_vector(
          spatial_file_path,
          layer = layer,
          returnclass = returnclass
        ))
      }
    }
  }

  cli::cli_inform("Reading {.file {basename(gdal_url)}} from naturalearth...")

  if (category == "raster") {
    # Try without subfolder first (TYPE.zip/TYPE.tif), then with subfolder
    # (TYPE.zip/TYPE/TYPE.tif) since Natural Earth zips have inconsistent
    # structure
    spatial_object <- tryCatch(
      suppressWarnings(terra::rast(gdal_url)),
      error = function(e) {
        type_name <- tools::file_path_sans_ext(basename(gdal_url))
        gdal_url_with_folder <- file.path(
          dirname(gdal_url),
          type_name,
          basename(gdal_url)
        )
        terra::rast(gdal_url_with_folder)
      }
    )
  } else {
    layer <- layer_name(type, scale)
    spatial_object <- read_spatial_vector(
      gdal_url,
      layer = layer,
      returnclass = returnclass
    )
  }

  if (load) {
    # If using cache directory, save a copy for future use
    if (use_cache) {
      spatial_file_path <- make_dest_path(gdal_url, category, destdir)

      cli::cli_inform(
        "Writing {.file {basename(spatial_file_path)}} to {.path {destdir}}..."
      )

      if (category == "raster") {
        terra::writeRaster(
          spatial_object,
          spatial_file_path,
          overwrite = TRUE,
          gdal = c(
            "COMPRESS=ZSTD",
            "PREDICTOR=2",
            "TILED=YES",
            "TFW=NO",
            "BLOCKXSIZE=256",
            "BLOCKYSIZE=256"
          )
        )
      } else if (returnclass == "sf") {
        sf::write_sf(spatial_object, spatial_file_path, delete_dsn = TRUE)
      } else {
        terra::writeVector(spatial_object, spatial_file_path, overwrite = TRUE)
      }
    }

    return(spatial_object)
  }

  spatial_file_path <- make_dest_path(gdal_url, category, destdir)

  cli::cli_inform(
    "Writing {.file {basename(spatial_file_path)}} to {.path {destdir}}..."
  )

  if (category == "raster") {
    terra::writeRaster(
      spatial_object,
      spatial_file_path,
      overwrite = TRUE,
      gdal = c(
        "COMPRESS=ZSTD",
        "PREDICTOR=2",
        "TILED=YES",
        "TFW=NO",
        "BLOCKXSIZE=256",
        "BLOCKYSIZE=256"
      )
    )
  } else if (returnclass == "sf") {
    sf::write_sf(spatial_object, spatial_file_path, delete_dsn = TRUE)
  } else {
    terra::writeVector(spatial_object, spatial_file_path, overwrite = TRUE)
  }

  invisible(spatial_file_path)
}

test_that("ne_download() can download data and return the associated file when load = FALSE", {
  skip_on_cran()
  skip_if_not_installed("rnaturalearthdata")
  skip_if_not_installed("rnaturalearthhires")

  # fmt: skip
  urls <- c(
    ne_download( scale = 110L, type = "countries", category = "cultural", load = FALSE),
    ne_download(scale = 110L, type = "map_units", category = "cultural", load = FALSE),
    ne_download(scale = 110L, type = "sovereignty", category = "cultural", load = FALSE),
    ne_download(scale = 50L, type = "countries", category = "cultural", load = FALSE),
    ne_download(scale = 10L, type = "countries", category = "cultural", load = FALSE),
    ne_download(scale = 10L, type = "map_units", category = "cultural", load = FALSE),
    ne_download(scale = 10L, type = "sovereignty", category = "cultural", load = FALSE),
    ne_download(scale = 10L, type = "coastline", category = "physical", load = FALSE),
    ne_download(scale = 10L, type = "parks_and_protected_lands_line", category = "cultural", load = FALSE),
    ne_download(scale = 10L, type = "parks_and_protected_lands_point", category = "cultural", load = FALSE),
    ne_download(scale = 10L, type = "parks_and_protected_lands_scale_rank", category = "cultural", load = FALSE),
    ne_download(scale = 110L, type = "countries", category = "cultural", load = FALSE, returnclass = "sv"),
    ne_download(scale = 10L, type = "parks_and_protected_lands_scale_rank", category = "cultural", load = FALSE, returnclass = "sv"),
    ne_download(scale = 50L, type = "MSR_50M", category = "raster", load = FALSE)
  )

  expect_true(all(file.exists(urls)))
})

# fmt: on
test_that("ne_download() uses caching when load = TRUE and destdir is not tempdir()", {
  skip_on_cran()
  skip_if_not_installed("sf")

  # Create a temporary directory for caching
  cache_dir <- file.path(tempdir(), "ne_cache_test")
  dir.create(cache_dir, showWarnings = FALSE)
  on.exit(unlink(cache_dir, recursive = TRUE), add = TRUE)

  # First call: should download and save
  result1 <- ne_download(
    scale = 110L,
    type = "countries",
    category = "cultural",
    destdir = cache_dir,
    load = TRUE
  )

  # Check that the file was created
  expected_file <- file.path(cache_dir, "ne_110m_admin_0_countries.gpkg")
  expect_true(file.exists(expected_file))

  # Check that result is an sf object
  expect_s3_class(result1, "sf")

  # Second call: should load from cache (not re-download)
  result2 <- ne_download(
    scale = 110L,
    type = "countries",
    category = "cultural",
    destdir = cache_dir,
    load = TRUE
  )

  # Check that result is still an sf object
  expect_s3_class(result2, "sf")

  # Results should be identical
  expect_identical(nrow(result1), nrow(result2))
  expect_identical(ncol(result1), ncol(result2))
})

#' Get natural earth world coastline
#'
#' returns world coastline at specified scale
#'
#' @inherit ne_download
#'
#' @examples
#' if (requireNamespace("rnaturalearthdata")) {
#'   coast <- ne_coastline()
#'   plot(coast)
#' }
#'
#' @export
ne_coastline <- function(scale = 110L, returnclass = c("sf", "sv")) {
  returnclass <- match.arg(returnclass)

  if (returnclass == "sp") {
    deprecate_sp("ne_download(returnclass = 'sp')")
  }

  # check for the data packages and try to install if not there
  if (scale == 10L) {
    check_rnaturalearthhires()
  } else {
    check_rnaturalearthdata()
  }

  # check on permitted scales, convert names to numeric
  scale <- check_scale(scale)

  # choose which map based on scale
  sldf <- NULL

  if (scale == 110L) {
    sldf <- rnaturalearthdata::coastline110
  } else if (scale == 50L) {
    sldf <- rnaturalearthdata::coastline50
  } else if (scale == 10L) {
    sldf <- rnaturalearthhires::coastline10
  }

  # Convert to the desired class
  convert_spatial_class(sldf, returnclass)
}

---
title: "Language support in rnaturalearth"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Language support in rnaturalearth}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

This vignette shows how [rnaturalearth](https://github.com/ropensci/rnaturalearth) makes it easier to make maps with labels in languages other than Engish, and to make thematic maps if you have data that is referenced in languages other than Engish.

[rnaturalearth](https://github.com/ropensci/rnaturalearth) is an R package to hold and facilitate interaction with natural earth vector map data.

[Natural Earth](https://www.naturalearthdata.com/) is a public domain map dataset including vector country boundaries.

## Load required packages




``` r
library(rnaturalearth)
library(sf)
library(ggplot2)
library(ggrepel)
library(tmap)
library(knitr)

theme_set(theme_minimal())

sf_use_s2(FALSE)
```

## Country maps with labels in other languages


``` r
# Africa
terra::plot(ne_countries(continent = "africa"))
#> Warning: plotting the first 9 out of 168 attributes; use max.plot = 168 to plot all
```

![](language-support.Rmd-3-1.png)

``` r

sfaf <- ne_countries(continent = "africa", returnclass = "sf")
sfafc <- st_centroid(sfaf)
#> Warning: st_centroid assumes attributes are constant over geometries
#> Warning in st_centroid.sfc(st_geometry(x), of_largest_polygon = of_largest_polygon):
#> st_centroid does not give correct centroids for longitude/latitude data

sfaf <- cbind(sfaf, st_coordinates(st_centroid(sfaf$geometry)))
#> Warning in st_centroid.sfc(sfaf$geometry): st_centroid does not give correct centroids for
#> longitude/latitude data

# this adds centroids in the middle of countries
ggplot(sfaf) +
  geom_sf() +
  geom_sf(data = sfafc)
```

![](language-support.Rmd-3-2.png)

``` r

# trying labels in the middle of countries, doesn't quite work needs x,y,label
# but once x & y added on with st_coordinates ... seems getting the coords might
# not be necessary for much longer https://github.com/slowkow/ggrepel/issues/111
# cool nearly there ...
ggplot(sfaf) +
  geom_sf() +
  geom_text_repel(aes(x = X, y = Y, label = name), size = 3L)
```

![](language-support.Rmd-3-3.png)

``` r

# getting there, labels still overlap a bit maybe make map bigger to allow space
# for labels
ggplot(sfaf) +
  geom_sf() +
  geom_text_repel(aes(x = X, y = Y, label = name_es), size = 3L)
```

![](language-support.Rmd-3-4.png)

``` r

# point.padding=NA allows labels to overlap the centroid
ggplot(sfaf) +
  geom_sf() +
  geom_text_repel(
    aes(x = X, y = Y, label = name_es),
    point.padding = NA,
    size = 3L
  )
```

![](language-support.Rmd-3-5.png)

``` r

# Africa labels just down left & right sides works pretty well I think
ggplot(sfaf) +
  geom_sf() +
  xlim(-28L, 61L) +
  geom_text_repel(
    aes(x = X, y = Y, label = name_es),
    data = subset(sfaf, X > 21L),
    nudge_x = 60L - subset(sfaf, X > 21L)$X,
    segment.size = 0.2,
    segment.color = "grey50",
    direction = "y",
    hjust = 0L,
    size = 3L
  ) +
  geom_text_repel(
    aes(x = X, y = Y, label = name_es),
    data = subset(sfaf, X < 21L),
    nudge_x = -19L - subset(sfaf, X < 21L)$X,
    segment.size = 0.2,
    segment.color = "grey50",
    direction = "y",
    hjust = 1L,
    size = 3L
  )
```

![](language-support.Rmd-3-6.png)

``` r

# french labels
ggplot(sfaf) +
  geom_sf() +
  xlim(-28L, 61L) +
  geom_text_repel(
    aes(x = X, y = Y, label = name_fr),
    data = subset(sfaf, X > 21L),
    nudge_x = 60L - subset(sfaf, X > 21L)$X,
    segment.size = 0.2,
    segment.color = "grey50",
    direction = "y",
    hjust = 0L,
    size = 3L
  ) +
  geom_text_repel(
    aes(x = X, y = Y, label = name_fr),
    data = subset(sfaf, X < 21L),
    nudge_x = -19L - subset(sfaf, X < 21L)$X,
    segment.size = 0.2,
    segment.color = "grey50",
    direction = "y",
    hjust = 1L,
    size = 3L
  )
```

![](language-support.Rmd-3-7.png)

``` r

# tmap good but labels currently overlap

# english labels
tm_shape(sfaf) +
  tm_borders() +
  tm_text("name", size = 0.75)
```

![](language-support.Rmd-3-8.png)

``` r

# spanish labels
tm_shape(sfaf) +
  tm_borders() +
  tm_text("name_es", size = 0.75)
```

![](language-support.Rmd-3-9.png)

``` r

# other languages de, fr, nl,
tm_shape(sfaf) +
  tm_borders() +
  tm_text("name_de", size = 0.75)
```

![](language-support.Rmd-3-10.png)

---
title: "Introduction to rnaturalearth."
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to rnaturalearth.}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---



This vignette is an introduction to [rnaturalearth](https://github.com/ropensci/rnaturalearth), an R package to hold and facilitate interaction with natural earth vector map data. `rnaturalearth` is a data package designed to provide map data that can be visualised using other R packages.

[Natural Earth](https://www.naturalearthdata.com/) is a public domain map dataset including vector country and other administrative boundaries.

[rnaturalearth](https://github.com/ropensci/rnaturalearth) does two main things.

1. Contains pre-downloaded vector maps for :

   - countries `ne_countries()`
   - states `ne_states()`
   - coastline `ne_coastline()`

1. Has `ne_download()` function to facilitate download of other vector and raster maps.

This vignette uses `plot` as a simple, quick way to show how different data can be accessed.`rnaturalearth` is designed to provide data allowing creation of more elaborate maps in other visualisation packages (e.g. `ggplot2`, `tmap` and `choroplethr`).

## load required packages


``` r
library(rnaturalearth)
```

## Maps in the package.

Pre-downloaded maps can be accessed with :

- `ne_countries()` for country (admin-0) boundaries
- `ne_states()` for boundaries within countries (admin-1)
- `ne_coastline()` for world coastline


``` r
# world at small scale (low resolution)
plot(ne_countries(type = "countries", scale = "small"))
#> Warning: plotting the first 10 out of 168 attributes; use max.plot = 168 to plot all
```

![](rnaturalearth.Rmd-3-1.png)

``` r

# countries, UK undivided
plot(ne_countries(country = "united kingdom", type = "countries"))
#> Warning: plotting the first 9 out of 168 attributes; use max.plot = 168 to plot all
```

![](rnaturalearth.Rmd-3-2.png)

``` r
# map_units, UK divided into England, Scotland, Wales and Northern Ireland
plot(ne_countries(country = "united kingdom", type = "map_units"))
#> Warning: plotting the first 9 out of 168 attributes; use max.plot = 168 to plot all
```

![](rnaturalearth.Rmd-3-3.png)

``` r

# countries, small scale
plot(ne_countries(country = "united kingdom", scale = "small"))
#> Warning: plotting the first 9 out of 168 attributes; use max.plot = 168 to plot all
```

![](rnaturalearth.Rmd-3-4.png)

``` r

# countries, medium scale
plot(ne_countries(country = "united kingdom", scale = "medium"))
#> Warning: plotting the first 10 out of 168 attributes; use max.plot = 168 to plot all
```

![](rnaturalearth.Rmd-3-5.png)


``` r
# not evaluated because rely on rnaturalearthhires data which are on rOpenSci
# so CRAN check likely to fail

# countries, large scale
plot(ne_countries(country = "united kingdom", scale = "large"))

# states country='united kingdom'
plot(ne_states(country = "united kingdom"))
# states geounit='england'
plot(ne_states(geounit = "england"))

# states country='france'
plot(ne_states(country = "france"))
```


``` r
# coastline of the world
# subsetting of coastline is not possible because the Natural Earth data are
# not attributed in that way
plot(ne_coastline())
```

![](rnaturalearth.Rmd-5-1.png)

## Downloading other Natural Earth vectors with ne_download().

Each [Natural Earth](https://www.naturalearthdata.com/) dataset is characterised on the website according to `scale`, `type` and `category`. [rnaturalearth](https://github.com/ropensci/rnaturalearth) allows you to specify `scale`, `type` and `category` and will construct the url and download the corresponding file.


``` r
# lakes
lakes110 <- ne_download(scale = 110L, type = "lakes", category = "physical")
plot(lakes110, col = "blue")

# rivers
rivers110 <- ne_download(
  scale = 110L,
  type = "rivers_lake_centerlines",
  category = "physical"
)
plot(rivers110, col = "blue")
```

## Tables of vector layers available via `ne_download(type=[layer_name], scale=)`

1=available, 0=not



Table: category='physical' vector data available via ne_download()

|layer                              | scale10| scale50| scale110|
|:----------------------------------|-------:|-------:|--------:|
|antarctic_ice_shelves_lines        |       1|       1|        0|
|antarctic_ice_shelves_polys        |       1|       1|        0|
|coastline                          |       1|       1|        1|
|geographic_lines                   |       1|       1|        1|
|geography_marine_polys             |       1|       1|        1|
|geography_regions_elevation_points |       1|       1|        1|
|geography_regions_points           |       1|       1|        1|
|geography_regions_polys            |       1|       1|        1|
|glaciated_areas                    |       1|       1|        1|
|lakes                              |       1|       1|        1|
|lakes_europe                       |       1|       0|        0|
|lakes_historic                     |       1|       1|        0|
|lakes_north_america                |       1|       0|        0|
|lakes_pluvial                      |       1|       0|        0|
|land                               |       1|       1|        1|
|land_ocean_label_points            |       1|       0|        0|
|land_ocean_seams                   |       1|       0|        0|
|land_scale_rank                    |       1|       0|        0|
|minor_islands                      |       1|       0|        0|
|minor_islands_coastline            |       1|       0|        0|
|minor_islands_label_points         |       1|       0|        0|
|ocean                              |       1|       1|        1|
|ocean_scale_rank                   |       1|       0|        0|
|playas                             |       1|       1|        0|
|reefs                              |       1|       0|        0|
|rivers_europe                      |       1|       0|        0|
|rivers_lake_centerlines            |       1|       1|        1|
|rivers_lake_centerlines_scale_rank |       1|       1|        0|
|rivers_north_america               |       1|       0|        0|





Table: category='cultural' vector data available via ne_download()

|layer                                           | scale10| scale50| scale110|
|:-----------------------------------------------|-------:|-------:|--------:|
|admin_0_antarctic_claim_limit_lines             |       1|       0|        0|
|admin_0_antarctic_claims                        |       1|       0|        0|
|admin_0_boundary_lines_disputed_areas           |       1|       1|        0|
|admin_0_boundary_lines_land                     |       1|       1|        1|
|admin_0_boundary_lines_map_units                |       1|       0|        0|
|admin_0_boundary_lines_maritime_indicator       |       1|       1|        0|
|admin_0_boundary_map_units                      |       0|       1|        0|
|admin_0_breakaway_disputed_areas                |       0|       1|        0|
|admin_0_countries                               |       1|       1|        1|
|admin_0_countries_lakes                         |       1|       1|        1|
|admin_0_disputed_areas                          |       1|       0|        0|
|admin_0_disputed_areas_scale_rank_minor_islands |       1|       0|        0|
|admin_0_label_points                            |       1|       0|        0|
|admin_0_map_subunits                            |       1|       1|        0|
|admin_0_map_units                               |       1|       1|        1|
|admin_0_pacific_groupings                       |       1|       1|        1|
|admin_0_scale_rank                              |       1|       1|        1|
|admin_0_scale_rank_minor_islands                |       1|       0|        0|
|admin_0_seams                                   |       1|       0|        0|
|admin_0_sovereignty                             |       1|       1|        1|
|admin_0_tiny_countries                          |       0|       1|        1|
|admin_0_tiny_countries_scale_rank               |       0|       1|        0|
|admin_1_label_points                            |       1|       0|        0|
|admin_1_seams                                   |       1|       0|        0|
|admin_1_states_provinces                        |       1|       1|        1|
|admin_1_states_provinces_lakes                  |       1|       1|        1|
|admin_1_states_provinces_lines                  |       1|       1|        1|
|admin_1_states_provinces_scale_rank             |       1|       1|        1|
|airports                                        |       1|       1|        0|
|parks_and_protected_lands_area                  |       1|       0|        0|
|parks_and_protected_lands_line                  |       1|       0|        0|
|parks_and_protected_lands_point                 |       1|       0|        0|
|parks_and_protected_lands_scale_rank            |       1|       0|        0|
|populated_places                                |       1|       1|        1|
|populated_places_simple                         |       1|       1|        1|
|ports                                           |       1|       1|        0|
|railroads                                       |       1|       0|        0|
|railroads_north_america                         |       1|       0|        0|
|roads                                           |       1|       0|        0|
|roads_north_america                             |       1|       0|        0|
|time_zones                                      |       1|       0|        0|
|urban_areas                                     |       1|       1|        0|
|urban_areas_landscan                            |       1|       0|        0|

---
title: "Finding data"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Finding data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---




``` r
library(rnaturalearth)
library(sf)
#> Linking to GEOS 3.12.1, GDAL 3.8.5, PROJ 9.4.0; sf_use_s2() is TRUE
```

## Available data

There are a lot of data that can be downloaded from [Natural Earth](https://www.naturalearthdata.com/) with `ne_download()`. These data are divided into two main categories: _physical_ and _cultural_ vector data. The `df_layers_physical` and `df_layers_cultural` data frames included in the `rnaturalearth` packages show what layer of data can be downloaded.

### Physical vector data


``` r
data(df_layers_physical)

knitr::kable(
  df_layers_physical,
  caption = "physical vector data available via ne_download()"
)
```



Table: physical vector data available via ne_download()

|layer                              | scale10| scale50| scale110|
|:----------------------------------|-------:|-------:|--------:|
|antarctic_ice_shelves_lines        |       1|       1|        0|
|antarctic_ice_shelves_polys        |       1|       1|        0|
|coastline                          |       1|       1|        1|
|geographic_lines                   |       1|       1|        1|
|geography_marine_polys             |       1|       1|        1|
|geography_regions_elevation_points |       1|       1|        1|
|geography_regions_points           |       1|       1|        1|
|geography_regions_polys            |       1|       1|        1|
|glaciated_areas                    |       1|       1|        1|
|lakes                              |       1|       1|        1|
|lakes_europe                       |       1|       0|        0|
|lakes_historic                     |       1|       1|        0|
|lakes_north_america                |       1|       0|        0|
|lakes_pluvial                      |       1|       0|        0|
|land                               |       1|       1|        1|
|land_ocean_label_points            |       1|       0|        0|
|land_ocean_seams                   |       1|       0|        0|
|land_scale_rank                    |       1|       0|        0|
|minor_islands                      |       1|       0|        0|
|minor_islands_coastline            |       1|       0|        0|
|minor_islands_label_points         |       1|       0|        0|
|ocean                              |       1|       1|        1|
|ocean_scale_rank                   |       1|       0|        0|
|playas                             |       1|       1|        0|
|reefs                              |       1|       0|        0|
|rivers_europe                      |       1|       0|        0|
|rivers_lake_centerlines            |       1|       1|        1|
|rivers_lake_centerlines_scale_rank |       1|       1|        0|
|rivers_north_america               |       1|       0|        0|



Based on the previous table, we know that we can download the `ocean` vector at small scale (110). Note that scales are defined as one of `110`, `50`, `10` or `small`, `medium`, `large`.


``` r
plot(
  ne_download(type = "ocean", category = "physical", scale = "small")[
    "geometry"
  ],
  col = "lightblue"
)
#> Reading 'ne_110m_ocean.zip' from naturalearth...
```

![](finding-data.Rmd-3-1.png)

### Cultural vector data


``` r
data(df_layers_cultural)

knitr::kable(
  df_layers_cultural,
  caption = "cultural vector data available via ne_download()"
)
```



Table: cultural vector data available via ne_download()

|layer                                           | scale10| scale50| scale110|
|:-----------------------------------------------|-------:|-------:|--------:|
|admin_0_antarctic_claim_limit_lines             |       1|       0|        0|
|admin_0_antarctic_claims                        |       1|       0|        0|
|admin_0_boundary_lines_disputed_areas           |       1|       1|        0|
|admin_0_boundary_lines_land                     |       1|       1|        1|
|admin_0_boundary_lines_map_units                |       1|       0|        0|
|admin_0_boundary_lines_maritime_indicator       |       1|       1|        0|
|admin_0_boundary_map_units                      |       0|       1|        0|
|admin_0_breakaway_disputed_areas                |       0|       1|        0|
|admin_0_countries                               |       1|       1|        1|
|admin_0_countries_lakes                         |       1|       1|        1|
|admin_0_disputed_areas                          |       1|       0|        0|
|admin_0_disputed_areas_scale_rank_minor_islands |       1|       0|        0|
|admin_0_label_points                            |       1|       0|        0|
|admin_0_map_subunits                            |       1|       1|        0|
|admin_0_map_units                               |       1|       1|        1|
|admin_0_pacific_groupings                       |       1|       1|        1|
|admin_0_scale_rank                              |       1|       1|        1|
|admin_0_scale_rank_minor_islands                |       1|       0|        0|
|admin_0_seams                                   |       1|       0|        0|
|admin_0_sovereignty                             |       1|       1|        1|
|admin_0_tiny_countries                          |       0|       1|        1|
|admin_0_tiny_countries_scale_rank               |       0|       1|        0|
|admin_1_label_points                            |       1|       0|        0|
|admin_1_seams                                   |       1|       0|        0|
|admin_1_states_provinces                        |       1|       1|        1|
|admin_1_states_provinces_lakes                  |       1|       1|        1|
|admin_1_states_provinces_lines                  |       1|       1|        1|
|admin_1_states_provinces_scale_rank             |       1|       1|        1|
|airports                                        |       1|       1|        0|
|parks_and_protected_lands_area                  |       1|       0|        0|
|parks_and_protected_lands_line                  |       1|       0|        0|
|parks_and_protected_lands_point                 |       1|       0|        0|
|parks_and_protected_lands_scale_rank            |       1|       0|        0|
|populated_places                                |       1|       1|        1|
|populated_places_simple                         |       1|       1|        1|
|ports                                           |       1|       1|        0|
|railroads                                       |       1|       0|        0|
|railroads_north_america                         |       1|       0|        0|
|roads                                           |       1|       0|        0|
|roads_north_america                             |       1|       0|        0|
|time_zones                                      |       1|       0|        0|
|urban_areas                                     |       1|       1|        0|
|urban_areas_landscan                            |       1|       0|        0|




``` r
plot(
  ne_download(
    type = "airports",
    category = "cultural",
    scale = 10L
  )["geometry"],
  pch = 21L,
  bg = "grey"
)
#> Reading 'ne_10m_airports.zip' from naturalearth...
```

![](finding-data.Rmd-5-1.png)

## Searching for countries and continents

In this article, we explore how we can search for data available to download within `rnaturalearth`. Let's begin by loading country data using the `read_sf()` function from the `sf` package. In the following code snippet, we read the Natural Earth dataset, which contains information about the sovereignty of countries.


``` r
df <- read_sf(
  "/vsizip/vsicurl/https://naciscdn.org/naturalearth/10m/cultural/ne_10m_admin_0_sovereignty.zip"
)

head(df)
#> Simple feature collection with 6 features and 168 fields
#> Geometry type: MULTIPOLYGON
#> Dimension:     XY
#> Bounding box:  xmin: -109.4537 ymin: -55.9185 xmax: 140.9776 ymax: 7.35578
#> Geodetic CRS:  WGS 84
#> # A tibble: 6 × 169
#>   featurecla     scalerank LABELRANK SOVEREIGNT SOV_A3 ADM0_DIF LEVEL TYPE  TLC   ADMIN ADM0_A3
#>   <chr>              <int>     <int> <chr>      <chr>     <int> <int> <chr> <chr> <chr> <chr>  
#> 1 Admin-0 sover…         5         2 Indonesia  IDN           0     2 Sove… 1     Indo… IDN    
#> 2 Admin-0 sover…         5         3 Malaysia   MYS           0     2 Sove… 1     Mala… MYS    
#> 3 Admin-0 sover…         0         2 Chile      CHL           0     2 Sove… 1     Chile CHL    
#> 4 Admin-0 sover…         0         3 Bolivia    BOL           0     2 Sove… 1     Boli… BOL    
#> 5 Admin-0 sover…         0         2 Peru       PER           0     2 Sove… 1     Peru  PER    
#> 6 Admin-0 sover…         0         2 Argentina  ARG           0     2 Sove… 1     Arge… ARG    
#> # ℹ 158 more variables: GEOU_DIF <int>, GEOUNIT <chr>, GU_A3 <chr>, SU_DIF <int>,
#> #   SUBUNIT <chr>, SU_A3 <chr>, BRK_DIFF <int>, NAME <chr>, NAME_LONG <chr>, BRK_A3 <chr>,
#> #   BRK_NAME <chr>, BRK_GROUP <chr>, ABBREV <chr>, POSTAL <chr>, FORMAL_EN <chr>,
#> #   FORMAL_FR <chr>, NAME_CIAWF <chr>, NOTE_ADM0 <chr>, NOTE_BRK <chr>, NAME_SORT <chr>,
#> #   NAME_ALT <chr>, MAPCOLOR7 <int>, MAPCOLOR8 <int>, MAPCOLOR9 <int>, MAPCOLOR13 <int>,
#> #   POP_EST <dbl>, POP_RANK <int>, POP_YEAR <int>, GDP_MD <int>, GDP_YEAR <int>,
#> #   ECONOMY <chr>, INCOME_GRP <chr>, FIPS_10 <chr>, ISO_A2 <chr>, ISO_A2_EH <chr>,
#> #   ISO_A3 <chr>, ISO_A3_EH <chr>, ISO_N3 <chr>, ISO_N3_EH <chr>, UN_A3 <chr>, WB_A2 <chr>,
#> #   WB_A3 <chr>, WOE_ID <int>, WOE_ID_EH <int>, WOE_NOTE <chr>, ADM0_ISO <chr>,
#> #   ADM0_DIFF <chr>, ADM0_TLC <chr>, ADM0_A3_US <chr>, ADM0_A3_FR <chr>, ADM0_A3_RU <chr>,
#> #   ADM0_A3_ES <chr>, ADM0_A3_CN <chr>, ADM0_A3_TW <chr>, ADM0_A3_IN <chr>, ADM0_A3_NP <chr>,
#> #   ADM0_A3_PK <chr>, ADM0_A3_DE <chr>, ADM0_A3_GB <chr>, ADM0_A3_BR <chr>, ADM0_A3_IL <chr>,
#> #   ADM0_A3_PS <chr>, ADM0_A3_SA <chr>, ADM0_A3_EG <chr>, ADM0_A3_MA <chr>, ADM0_A3_PT <chr>,
#> #   ADM0_A3_AR <chr>, ADM0_A3_JP <chr>, ADM0_A3_KO <chr>, ADM0_A3_VN <chr>, ADM0_A3_TR <chr>,
#> #   ADM0_A3_ID <chr>, ADM0_A3_PL <chr>, ADM0_A3_GR <chr>, ADM0_A3_IT <chr>, ADM0_A3_NL <chr>,
#> #   ADM0_A3_SE <chr>, ADM0_A3_BD <chr>, ADM0_A3_UA <chr>, ADM0_A3_UN <int>, ADM0_A3_WB <int>,
#> #   CONTINENT <chr>, REGION_UN <chr>, SUBREGION <chr>, REGION_WB <chr>, NAME_LEN <int>,
#> #   LONG_LEN <int>, ABBREV_LEN <int>, TINY <int>, HOMEPART <int>, MIN_ZOOM <dbl>,
#> #   MIN_LABEL <dbl>, MAX_LABEL <dbl>, LABEL_X <dbl>, LABEL_Y <dbl>, NE_ID <dbl>,
#> #   WIKIDATAID <chr>, NAME_AR <chr>, NAME_BN <chr>, NAME_DE <chr>, …
```

### Finding countries

One way to search for countries is to search within the `ADMIN` vector. Let's start by plotting some of the first countries.


``` r

lapply(
  df$ADMIN[1L:6L],
  \(x) plot(ne_countries(country = x)["geometry"], main = x)
)
```

<img src="finding-data.Rmd-7-1.png" width="50%" /><img src="finding-data.Rmd-7-2.png" width="50%" /><img src="finding-data.Rmd-7-3.png" width="50%" /><img src="finding-data.Rmd-7-4.png" width="50%" /><img src="finding-data.Rmd-7-5.png" width="50%" /><img src="finding-data.Rmd-7-6.png" width="50%" />

Suppose that we want to search the polygons for the US, how should we spell it?


``` r
ne_countries(country = "USA")
ne_countries(country = "United States")
ne_countries(country = "United States Of America")
ne_countries(country = "United States of America")
```

One possibility consists to search within the `ADMIN` vector using a regular expression to find all occurrences of the word _states_.


``` r
grep("states", df$ADMIN, ignore.case = TRUE, value = TRUE)
#> [1] "United States of America"       "Federated States of Micronesia"
```

We can now get the data.


``` r
plot(ne_countries(country = "United States of America")["geometry"])
```

![](finding-data.Rmd-10-1.png)

### Continents

Finally, let's create plots for each continent using the `ne_countries` function with the continent parameter.


``` r
unique(df$CONTINENT)
#> [1] "Asia"                    "South America"           "Europe"                 
#> [4] "Africa"                  "North America"           "Oceania"                
#> [7] "Antarctica"              "Seven seas (open ocean)"
```


``` r

lapply(
  unique(df$CONTINENT),
  \(x)
    plot(
      ne_countries(
        continent = x,
        scale = "medium"
      )["geometry"],
      main = x
    )
)
```

<img src="finding-data.Rmd-12-1.png" width="50%" /><img src="finding-data.Rmd-12-2.png" width="50%" /><img src="finding-data.Rmd-12-3.png" width="50%" /><img src="finding-data.Rmd-12-4.png" width="50%" /><img src="finding-data.Rmd-12-5.png" width="50%" /><img src="finding-data.Rmd-12-6.png" width="50%" /><img src="finding-data.Rmd-12-7.png" width="50%" /><img src="finding-data.Rmd-12-8.png" width="50%" />

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{convert_spatial_class}
\alias{convert_spatial_class}
\title{Convert from/to sf/sv objects}
\usage{
convert_spatial_class(x, returnclass = c("sf", "sv"))
}
\arguments{
\item{x}{Object to be converted}

\item{returnclass}{A string determining the spatial object to return. Either
"sf" for for simple feature (from `sf`, the default) or "sv" for a
`SpatVector` (from `terra`).}
}
\value{
Object of class "sf" or "sv"
}
\description{
returns downloaded data as a spatial object or the filename if
\code{load=FALSE}. if \code{destdir} is specified the data can be reloaded in
a later R session using \code{\link{ne_load}} with the same arguments.
}
\details{
Note that the filename of the requested object will be returned if
`load = FALSE`.

If the data is to be loaded into memory (`load = TRUE`), the download will
be handled using the GDAL virtual file system, allowing direct access to the
data without writing it to disk.
}
\examples{
\dontrun{
spdf_world <- ne_download(scale = 110, type = "countries")

plot(spdf_world)
plot(ne_download(type = "populated_places"))

# reloading from the saved file in the same session with same arguments

spdf_world2 <- ne_load(scale = 110, type = "countries")

# download followed by load from specified directory will work across sessions
spdf_world <- ne_download(scale = 110, type = "countries", destdir = getwd())
spdf_world2 <- ne_load(scale = 110, type = "countries", destdir = getwd())

# for raster, here an example with Manual Shaded Relief (MSR) download & load

rst <- ne_download(scale = 50, type = "MSR_50M", category = "raster", destdir = getwd())

# load after having downloaded
rst <- ne_load(
  scale = 50, type = "MSR_50M", category = "raster", destdir = getwd()
)

# plot
library(terra)
terra::plot(rst)
# end dontrun
}

}
\seealso{
\code{\link{ne_load}}, pre-downloaded data are available using
  \code{\link{ne_countries}}, \code{\link{ne_states}}. Other geographic data
  are available in the raster package : \code{\link[raster]{getData}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ne_coastline.R
\name{ne_coastline}
\alias{ne_coastline}
\title{Get natural earth world coastline}
\usage{
ne_coastline(scale = 110L, returnclass = c("sf", "sv"))
}
\arguments{
\item{scale}{The scale of map to return, one of `110`, `50`, `10` or `small`,
`medium`, `large`.}

\item{returnclass}{A string determining the spatial object to return. Either
"sf" for for simple feature (from `sf`, the default) or "sv" for a
`SpatVector` (from `terra`).}
}
\value{
An object of class `sf` for simple feature (from `sf`, the default)
or `SpatVector` (from `terra`).
}
\description{
returns world coastline at specified scale
}
\details{
Note that the filename of the requested object will be returned if
`load = FALSE`.

If the data is to be loaded into memory (`load = TRUE`), the download will
be handled using the GDAL virtual file system, allowing direct access to the
data without writing it to disk.
}
\examples{
if (requireNamespace("rnaturalearthdata")) {
  coast <- ne_coastline()
  plot(coast)
}

}
\seealso{
\code{\link{ne_load}}, pre-downloaded data are available using
  \code{\link{ne_countries}}, \code{\link{ne_states}}. Other geographic data
  are available in the raster package : \code{\link[raster]{getData}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_scale.R
\name{check_scale}
\alias{check_scale}
\title{check that this scale is present in Natural Earth}
\usage{
check_scale(x)
}
\arguments{
\item{x}{scale of map to return, one of \code{110}, \code{50}, \code{10} or
\code{'small'}, \code{'medium'}, \code{'large'}}
}
\value{
integer scale of map
}
\description{
check name or numeric scale representations, return numeric one
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{make_dest_path}
\alias{make_dest_path}
\title{Create Destination File Path}
\usage{
make_dest_path(gdal_url, category, destdir)
}
\arguments{
\item{gdal_url}{A character string representing the GDAL URL}

\item{category}{A character string specifying the data category ("raster" or other)}

\item{destdir}{A character string specifying the destination directory}
}
\value{
A character string representing the complete destination file path
}
\description{
Creates a destination file path by combining the directory path
with a formatted filename based on the GDAL URL and category.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_data.R
\name{ne_git_contents}
\alias{ne_git_contents}
\title{Return contents of Natural Earth Github directory}
\usage{
ne_git_contents(path)
}
\arguments{
\item{path}{string, one of: \code{'110m_physical'}, \code{'110m_cultural'},
\code{'50m_physical'}, \code{'50m_cultural'}, \code{'10m_physical'},
\code{'10m_cultural'}}
}
\value{
list. Includes parsed json content, http path, and response
  code.
}
\description{
Uses the Github API to return contents of Natural Earth Github directories.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/install-rnaturalearthdata.R
\name{install_rnaturalearthdata}
\alias{install_rnaturalearthdata}
\title{Install the naturalearthdata package after checking with the user}
\usage{
install_rnaturalearthdata()
}
\description{
Install the naturalearthdata package after checking with the user
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_data.R
\name{ne_git_layer_names}
\alias{ne_git_layer_names}
\title{Create list of layer names and metadata links}
\usage{
ne_git_layer_names(x, scale, getmeta)
}
\arguments{
\item{x}{object returned by ne_git_contents}

\item{scale}{one of \code{110}, \code{50}, \code{10}}

\item{getmeta}{whether to get url of the metadata for each layer}
}
\value{
list of lists with layer names and metadata links.
}
\description{
Parses Natural Earth Github folder content for layer names and metadata
links.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_data.R
\name{get_data}
\alias{get_data}
\title{Get data from within the package}
\usage{
get_data(
  scale = 110L,
  type = c("countries", "map_units", "sovereignty", "tiny_countries")
)
}
\arguments{
\item{scale}{The scale of map to return, one of `110`, `50`, `10` or `small`,
`medium`, `large`.}

\item{type}{country type, one of 'countries', 'map_units', 'sovereignty',
'tiny_countries'}
}
\value{
A \code{sf} object.
}
\description{
returns world country polygons at a specified scale, used by ne_countries()
}
\details{
Note that the filename of the requested object will be returned if
`load = FALSE`.

If the data is to be loaded into memory (`load = TRUE`), the download will
be handled using the GDAL virtual file system, allowing direct access to the
data without writing it to disk.
}
\examples{
\dontrun{
spdf_world <- ne_download(scale = 110, type = "countries")

plot(spdf_world)
plot(ne_download(type = "populated_places"))

# reloading from the saved file in the same session with same arguments

spdf_world2 <- ne_load(scale = 110, type = "countries")

# download followed by load from specified directory will work across sessions
spdf_world <- ne_download(scale = 110, type = "countries", destdir = getwd())
spdf_world2 <- ne_load(scale = 110, type = "countries", destdir = getwd())

# for raster, here an example with Manual Shaded Relief (MSR) download & load

rst <- ne_download(scale = 50, type = "MSR_50M", category = "raster", destdir = getwd())

# load after having downloaded
rst <- ne_load(
  scale = 50, type = "MSR_50M", category = "raster", destdir = getwd()
)

# plot
library(terra)
terra::plot(rst)
# end dontrun
}

}
\seealso{
\code{\link{ne_load}}, pre-downloaded data are available using
  \code{\link{ne_countries}}, \code{\link{ne_states}}. Other geographic data
  are available in the raster package : \code{\link[raster]{getData}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{df_layers_physical}
\alias{df_layers_physical}
\title{list of physical layers available from Natural Earth}
\format{
A \code{DataFrame}

An object of class \code{data.frame} with 29 rows and 4 columns.
}
\usage{
df_layers_physical
}
\description{
list of physical layers available from Natural Earth
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_data_exist.R
\name{check_data_exist}
\alias{check_data_exist}
\title{check whether the requested data exist on Natural Earth}
\usage{
check_data_exist(
  type,
  scale = 110L,
  category = c("cultural", "physical", "raster")
)
}
\arguments{
\item{type}{type of natural earth file to download one of 'countries',
'map_units', 'map_subunits', 'sovereignty', 'states' OR the portion of any
natural earth vector url after the scale and before the . e.g. for
'ne_50m_urban_areas.zip' this would be 'urban_areas' OR the raster
filename e.g. for 'MSR_50M.zip' this would be 'MSR_50M'}

\item{scale}{The scale of map to return, one of `110`, `50`, `10` or `small`,
`medium`, `large`.}

\item{category}{one of natural earth categories : 'cultural', 'physical',
'raster'}
}
\value{
TRUE or FALSE
}
\description{
checks from a list dependent on type, category and scale. If it returns FALSE
the data may still exist on the website. Doesn't yet do checking on raster
names because I found the naming convention too tricky.
}
\details{
Note that the filename of the requested object will be returned if
`load = FALSE`.

If the data is to be loaded into memory (`load = TRUE`), the download will
be handled using the GDAL virtual file system, allowing direct access to the
data without writing it to disk.
}
\examples{
check_data_exist(type = "countries", scale = 110, category = "cultural")

# Type not in list for this category
check_data_exist(type = "airports", scale = 110, category = "physical")

# Type in list but scale shows FALSE
check_data_exist(type = "airports", scale = 110, category = "cultural")

}
\seealso{
\code{\link{ne_load}}, pre-downloaded data are available using
  \code{\link{ne_countries}}, \code{\link{ne_states}}. Other geographic data
  are available in the raster package : \code{\link[raster]{getData}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{read_spatial_vector}
\alias{read_spatial_vector}
\title{Read Spatial Vector}
\usage{
read_spatial_vector(x, layer, returnclass = c("sf", "sv"))
}
\arguments{
\item{x}{A character string specifying the path to the spatial vector file.}

\item{layer}{A character string specifying the later to read in case there
are more than one in the zip file.}

\item{returnclass}{A character string specifying the class of object to
return. Options are "sf" for sf object and "sv" for sv object.}
}
\value{
Either an `sf` object or a `sv` object.
}
\description{
This function reads a spatial vector file and returns either an `sf` object
or a `sv` object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ne_file_name.R
\name{normalize_type}
\alias{normalize_type}
\title{Normalize the type argument for Natural Earth datasets}
\usage{
normalize_type(type)
}
\arguments{
\item{type}{type of natural earth file to download one of 'countries',
'map_units', 'map_subunits', 'sovereignty', 'states' OR the portion of any
natural earth vector url after the scale and before the . e.g. for
'ne_50m_urban_areas.zip' this would be 'urban_areas' OR the raster
filename e.g. for 'MSR_50M.zip' this would be 'MSR_50M'}
}
\value{
A string representing the normalized dataset type.
}
\description{
This function standardizes the `type` argument by mapping common names to
their respective Natural Earth dataset names.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rnaturalearth-package.R
\docType{package}
\name{rnaturalearth-package}
\alias{rnaturalearth}
\alias{rnaturalearth-package}
\title{rnaturalearth: World Map Data from Natural Earth}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Facilitates mapping by making natural earth map data from \url{https://www.naturalearthdata.com/} more easily available to R users.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://docs.ropensci.org/rnaturalearth/}
  \item \url{https://github.com/ropensci/rnaturalearth}
  \item \url{https://docs.ropensci.org/rnaturalearthhires/}
  \item Report bugs at \url{https://github.com/ropensci/rnaturalearth/issues}
}

}
\author{
\strong{Maintainer}: Philippe Massicotte \email{pmassicotte@hotmail.com} (\href{https://orcid.org/0000-0002-5919-4116}{ORCID})

Authors:
\itemize{
  \item Andy South \email{southandy@gmail.com}
}

Other contributors:
\itemize{
  \item Koen Hufkens \email{koen.hufkens@gmail.com} (\href{https://orcid.org/0000-0002-5070-8109}{ORCID}) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{sanitize_gdal_url}
\alias{sanitize_gdal_url}
\title{Extracts the http URL from a VSIZIP URL}
\usage{
sanitize_gdal_url(url)
}
\arguments{
\item{url}{A character string representing the VSIZIP URL}
}
\value{
A character string representing the http URL extracted from the
VSIZIP URL
}
\description{
This function takes a VSIZIP URL and extracts the http URL from
it
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ne_file_name.R
\name{layer_name}
\alias{layer_name}
\title{Generate the layer name for a Natural Earth dataset}
\usage{
layer_name(type, scale)
}
\arguments{
\item{type}{type of natural earth file to download one of 'countries',
'map_units', 'map_subunits', 'sovereignty', 'states' OR the portion of any
natural earth vector url after the scale and before the . e.g. for
'ne_50m_urban_areas.zip' this would be 'urban_areas' OR the raster
filename e.g. for 'MSR_50M.zip' this would be 'MSR_50M'}

\item{scale}{The scale of map to return, one of `110`, `50`, `10` or `small`,
`medium`, `large`.}
}
\value{
A string representing the dataset layer name.
}
\description{
Generate the layer name for a Natural Earth dataset
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ne_states.R
\name{ne_states}
\alias{ne_states}
\alias{ne_admin1}
\title{Get natural earth world state (admin level 1) polygons}
\usage{
ne_states(
  country = NULL,
  geounit = NULL,
  iso_a2 = NULL,
  spat_object = NULL,
  returnclass = c("sf", "sv")
)
}
\arguments{
\item{country}{a character vector of country names.}

\item{geounit}{a character vector of geounit names.}

\item{iso_a2}{a character vector of iso_a2 country codes}

\item{spat_object}{an optional alternative states map}

\item{returnclass}{A string determining the spatial object to return. Either
"sf" for for simple feature (from `sf`, the default) or "sv" for a
`SpatVector` (from `terra`).}
}
\value{
An object of class `sf` for simple feature (from `sf`, the default)
or `SpatVector` (from `terra`).
}
\description{
returns state polygons (administrative level 1) for specified countries
}
\details{
By default, this function uses the scale = 10 data from the
rnaturalearthhires package. While data at scales 50 and 110 exist,
they are not used as defaults because they contain fewer countries
(e.g. scale 50 only includes Australia, Brazil, Canada and United
States of America). For finer control, use \code{\link{ne_download}}.

\code{ne_download( scale = 10L, type = "states", category = "cultural")}

\code{ne_download( scale = 50L, type = "states", category = "cultural")}

\code{ne_download( scale = 110L, type = "states", category = "cultural")}
}
\examples{

# comparing using country and geounit to filter
if (requireNamespace("rnaturalearthhires")) {
  spdf_france_country <- ne_states(country = "france")
  spdf_france_geounit <- ne_states(geounit = "france")

  plot(spdf_france_country)
  plot(spdf_france_geounit)

  plot(ne_states(country = "united kingdom"))
  plot(ne_states(geounit = "england"))
}

}
\seealso{
\code{\link{ne_load}}, pre-downloaded data are available using
  \code{\link{ne_countries}}, \code{\link{ne_states}}. Other geographic data
  are available in the raster package : \code{\link[raster]{getData}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ne_download.R
\name{ne_download}
\alias{ne_download}
\title{Download data from Natural Earth and (optionally) read into R}
\usage{
ne_download(
  scale = 110L,
  type = "countries",
  category = c("cultural", "physical", "raster"),
  destdir = tempdir(),
  load = TRUE,
  returnclass = c("sf", "sv")
)
}
\arguments{
\item{scale}{The scale of map to return, one of `110`, `50`, `10` or `small`,
`medium`, `large`.}

\item{type}{type of natural earth file to download one of 'countries',
'map_units', 'map_subunits', 'sovereignty', 'states' OR the portion of any
natural earth vector url after the scale and before the . e.g. for
'ne_50m_urban_areas.zip' this would be 'urban_areas'. See Details. OR the
raster filename e.g. for 'MSR_50M.zip' this would be 'MSR_50M'}

\item{category}{one of natural earth categories : 'cultural', 'physical',
'raster'}

\item{destdir}{where to save files, defaults to \code{tempdir()},
\code{getwd()} is also possible.}

\item{load}{`TRUE` to load the spatial object into R, `FALSE` to return the
filename of the downloaded object. If the requested object is a vector, it
will be saved as a GPKG file. If a raster is requested, it will be saved as
a GeoTIFF file.}

\item{returnclass}{A string determining the spatial object to return. Either
"sf" for for simple feature (from `sf`, the default) or "sv" for a
`SpatVector` (from `terra`).}
}
\value{
An object of class `sf` for simple feature (from `sf`, the default)
or `SpatVector` (from `terra`).
}
\description{
returns downloaded data as a spatial object or the filename if
\code{load=FALSE}. if \code{destdir} is specified the data can be reloaded in
a later R session using \code{\link{ne_load}} with the same arguments.
}
\details{
Note that the filename of the requested object will be returned if
`load = FALSE`.

If the data is to be loaded into memory (`load = TRUE`), the download will
be handled using the GDAL virtual file system, allowing direct access to the
data without writing it to disk.
}
\examples{
\dontrun{
spdf_world <- ne_download(scale = 110, type = "countries")

plot(spdf_world)
plot(ne_download(type = "populated_places"))

# reloading from the saved file in the same session with same arguments

spdf_world2 <- ne_load(scale = 110, type = "countries")

# download followed by load from specified directory will work across sessions
spdf_world <- ne_download(scale = 110, type = "countries", destdir = getwd())
spdf_world2 <- ne_load(scale = 110, type = "countries", destdir = getwd())

# for raster, here an example with Manual Shaded Relief (MSR) download & load

rst <- ne_download(scale = 50, type = "MSR_50M", category = "raster", destdir = getwd())

# load after having downloaded
rst <- ne_load(
  scale = 50, type = "MSR_50M", category = "raster", destdir = getwd()
)

# plot
library(terra)
terra::plot(rst)
# end dontrun
}

}
\seealso{
\code{\link{ne_load}}, pre-downloaded data are available using
  \code{\link{ne_countries}}, \code{\link{ne_states}}. Other geographic data
  are available in the raster package : \code{\link[raster]{getData}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{countries}
\alias{countries}
\alias{countries110}
\title{world country polygons from Natural Earth}
\format{
A \code{sf} object.

An object of class \code{sf} (inherits from \code{data.frame}) with 177 rows and 169 columns.
}
\source{
\url{https://naciscdn.org/naturalearth/10m/cultural/ne_10m_admin_0_countries.zip}
}
\usage{
countries110
}
\description{
at 1:110m scale (small). Other data and resolutions are in the packages
rnaturalearthdata and rnaturalearthhires.
}
\section{Slots}{

\describe{
\item{\code{data}}{A data frame with country attributes.}
}}

\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_data.R
\name{ne_find_vector_data}
\alias{ne_find_vector_data}
\title{Return a dataframe of available vector layers on Natural Earth}
\usage{
ne_find_vector_data(
  scale = 110L,
  category = c("cultural", "physical"),
  getmeta = FALSE
)
}
\arguments{
\item{scale}{The scale of map to return, one of `110`, `50`, `10` or `small`,
`medium`, `large`.}

\item{category}{one of natural earth categories : 'cultural', 'physical'}

\item{getmeta}{whether to get url of the metadata for each layer}
}
\value{
dataframe with two variables: layer and metadata
}
\description{
Checks the Natural Earth Github repository for current vector layers and
provides the file name required in the type argument of ne_download.
}
\details{
Note that the filename of the requested object will be returned if
`load = FALSE`.

If the data is to be loaded into memory (`load = TRUE`), the download will
be handled using the GDAL virtual file system, allowing direct access to the
data without writing it to disk.
}
\examples{
\dontrun{
ne_find_vector_data(scale = 10, category = "physical")
}
}
\seealso{
\code{\link{ne_load}}, pre-downloaded data are available using
  \code{\link{ne_countries}}, \code{\link{ne_states}}. Other geographic data
  are available in the raster package : \code{\link[raster]{getData}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ne_countries.R
\name{ne_countries}
\alias{ne_countries}
\alias{ne_admin0}
\title{Get natural earth world country polygons}
\usage{
ne_countries(
  scale = 110L,
  type = "countries",
  continent = NULL,
  country = NULL,
  geounit = NULL,
  sovereignty = NULL,
  returnclass = c("sf", "sv")
)
}
\arguments{
\item{scale}{The scale of map to return, one of `110`, `50`, `10` or `small`,
`medium`, `large`.}

\item{type}{country type, one of 'countries', 'map_units', 'sovereignty',
'tiny_countries'}

\item{continent}{a character vector of continent names to get countries from.}

\item{country}{a character vector of country names.}

\item{geounit}{a character vector of geounit names.}

\item{sovereignty}{a character vector of sovereignty names.}

\item{returnclass}{A string determining the spatial object to return. Either
"sf" for for simple feature (from `sf`, the default) or "sv" for a
`SpatVector` (from `terra`).}
}
\value{
An object of class `sf` for simple feature (from `sf`, the default)
or `SpatVector` (from `terra`).
}
\description{
returns world country polygons at a specified scale, or points of
tiny_countries
}
\details{
Note that the filename of the requested object will be returned if
`load = FALSE`.

If the data is to be loaded into memory (`load = TRUE`), the download will
be handled using the GDAL virtual file system, allowing direct access to the
data without writing it to disk.
}
\examples{
world <- ne_countries()
africa <- ne_countries(continent = "africa")
france <- ne_countries(country = "france")

plot(world$geometry)
plot(africa$geometry)
plot(france$geometry)

# get as SpatVector
world <- ne_countries(returnclass = "sv")
terra::plot(world)

tiny_countries <- ne_countries(type = "tiny_countries", scale = 50)
plot(tiny_countries)

}
\seealso{
\code{\link{ne_load}}, pre-downloaded data are available using
  \code{\link{ne_countries}}, \code{\link{ne_states}}. Other geographic data
  are available in the raster package : \code{\link[raster]{getData}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ne_file_name.R
\name{ne_file_name}
\alias{ne_file_name}
\title{return a natural earth filename based on arguments}
\usage{
ne_file_name(
  scale = 110L,
  type = "countries",
  category = c("cultural", "physical", "raster")
)
}
\arguments{
\item{scale}{The scale of map to return, one of `110`, `50`, `10` or `small`,
`medium`, `large`.}

\item{type}{type of natural earth file to download one of 'countries',
'map_units', 'map_subunits', 'sovereignty', 'states' OR the portion of any
natural earth vector url after the scale and before the . e.g. for
'ne_50m_urban_areas.zip' this would be 'urban_areas' OR the raster
filename e.g. for 'MSR_50M.zip' this would be 'MSR_50M'}

\item{category}{one of natural earth categories : 'cultural', 'physical',
'raster'}
}
\value{
string
}
\description{
returns a string that can then be used to download the file.
}
\details{
Note that the filename of the requested object will be returned if
`load = FALSE`.

If the data is to be loaded into memory (`load = TRUE`), the download will
be handled using the GDAL virtual file system, allowing direct access to the
data without writing it to disk.
}
\examples{
ne_url <- ne_file_name(scale = 110, type = "countries")

}
\seealso{
\code{\link{ne_load}}, pre-downloaded data are available using
  \code{\link{ne_countries}}, \code{\link{ne_states}}. Other geographic data
  are available in the raster package : \code{\link[raster]{getData}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ne_load.R
\name{ne_load}
\alias{ne_load}
\title{load a Natural Earth vector that has already been downloaded to R using
\code{\link{ne_download}}}
\usage{
ne_load(
  scale = 110L,
  type = "countries",
  category = c("cultural", "physical", "raster"),
  destdir = tempdir(),
  file_name = NULL,
  returnclass = c("sf", "sv")
)
}
\arguments{
\item{scale}{The scale of map to return, one of `110`, `50`, `10` or `small`,
`medium`, `large`.}

\item{type}{type of natural earth file one of 'countries', 'map_units',
'map_subunits', 'sovereignty', 'states' OR the portion of any natural
earth vector url after the scale and before the . e.g. for
'ne_50m_urban_areas.zip' this would be 'urban_areas' OR the raster
filename e.g. for 'MSR_50M.zip' this would be 'MSR_50M'}

\item{category}{one of natural earth categories : 'cultural', 'physical',
'raster'}

\item{destdir}{folder to load files from, default = tempdir()}

\item{file_name}{OPTIONAL name of file (excluding path) instead of natural
earth attributes}

\item{returnclass}{A string determining the spatial object to return. Either
"sf" for for simple feature (from `sf`, the default) or "sv" for a
`SpatVector` (from `terra`).}
}
\value{
An object of class `sf` for simple feature (from `sf`, the default)
or `SpatVector` (from `terra`).
}
\description{
returns loaded data as a spatial object.
}
\details{
This function should be used after first downloading the data with
\code{ne_download(load = FALSE)}. The downloaded file can then be loaded
using this function.
}
\examples{
\dontrun{
# download followed by load from tempdir() works in same R session
spdf_world <- ne_download(
  scale = 110,
  type = "countries"
)
spdf_world2 <- ne_load(
  scale = 110,
  type = "countries"
)

# download followed by load from specified directory works between R sessions
spdf_world <- ne_download(
  scale = 110,
  type = "countries",
  destdir = getwd()
)
spdf_world2 <- ne_load(
  scale = 110,
  type = "countries",
  destdir = getwd()
)

# for raster download & load
rst <- ne_download(
  scale = 50,
  type = "OB_50M",
  category = "raster",
  destdir = getwd(),
  load = FALSE
)

# load after having downloaded
rst <- ne_load(
  scale = 50,
  type = "OB_50M",
  category = "raster",
  destdir = getwd()
)

# plot
library(terra)
plot(rst)
# end dontrun
}

}
\seealso{
\code{\link{ne_download}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/install-rnaturalearthhires.R
\name{install_rnaturalearthhires}
\alias{install_rnaturalearthhires}
\title{Install the naturalearthhires package after checking with the user}
\usage{
install_rnaturalearthhires()
}
\description{
Install the naturalearthhires package after checking with the user
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/install-rnaturalearthdata.R
\name{check_rnaturalearthdata}
\alias{check_rnaturalearthdata}
\title{Check whether to install rnaturalearthdata and install if necessary}
\usage{
check_rnaturalearthdata()
}
\description{
If the rnaturalearthdata package is not installed, install it from GitHub
using pak. If it is not up to date, reinstall it.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/install-rnaturalearthhires.R
\name{check_rnaturalearthhires}
\alias{check_rnaturalearthhires}
\title{Check whether to install rnaturalearthhires and install if necessary}
\usage{
check_rnaturalearthhires()
}
\description{
If the rnaturalearthhires package is not installed, install it from GitHub
using pak. If it is not up to date, reinstall it.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{df_layers_cultural}
\alias{df_layers_cultural}
\title{list of cultural layers available from Natural Earth}
\format{
A \code{DataFrame}

An object of class \code{data.frame} with 43 rows and 4 columns.
}
\usage{
df_layers_cultural
}
\description{
list of cultural layers available from Natural Earth
}
\keyword{datasets}
